#ifndef API_LEM_H
#define API_LEM_H

/*******************************************************
 * TITLE: LEM.h
 * AUTHOR: Richard Whittaker
 * CREATION DATE: 1/6/03
 * PURPOSE:
 * This class is the primary access class for encoding
 * and language information.  This class is also used to 
 * obtain all externalized strings from resource files 
 * based on the string keys. Any use of this class must
 * be prefaced by a call to LEM::getInstance() to obtain
 * the singleton LEM object.
 *******************************************************/

#ifdef WIN32
#define PROP_FILE_SPEC "*.properties"
#else
#define PROP_FILE_SPEC ".properties"
#endif

#define MAX_LANG_LEN 32
#define MAX_ENC_LEN 32

#pragma warning(disable:4786)
#pragma warning(disable:4251)

//#include <vector>
#include <map>
#include <set>
#include "API_Message.h"
#include "API_StringMap.h"
#include "API_NeoLocale.h"
#include "API_MapKeys.h"
#include "NeoAPICriticalSection.h"

struct API_Lemming
{
	char language[MAX_LANG_LEN];
	char encoding[MAX_ENC_LEN];
};

typedef std::map<API_LanguageKey, API_NeoLocale*> NeoAPILangLocaleMap;
typedef std::set<API_LanguageKey> NeoAPILangKeySet;
typedef std::map<unsigned long, API_Lemming> NeoAPIThreadLemmingMap;

class API_LEM
{
public:

	API_LEM();
	API_LEM(const char * rootDir);

	~API_LEM();

	// getString returns an unformatted Message object
	API_FMessage getString(string_key key);


	// getInstance methods, return the lem_m pointer for use
	// anywhere in the server
	static API_LEM* getInstance() { return lem_m; }
	static API_LEM* getInstance(bool initflag);
	static API_LEM* getInstance(const char * rootDir);

	// updateThread sets current thread/lemming info, if doesn't exist, add new lemming to map
	void updateThread(const char * language, const char * encoding);
	
	// these methods will update the language or charset elements related to the current thread.
	// if it can't find the thread in the map it will NOT add an entry to the map for the thread
	void setLanguage(const char * language);
	void setCharset(const char * charset);

	const char* getLanguage();
	const char* getCharset();


	// returns initialized
	bool isInitialized();

	// sets the root directory
	void setRootDir(const char * newRD);
	
	// returns root directory
	static const char *rootDir();

	// check if the lem contains an entry for the current thread
	bool checkThread();

private:
	// lemmap_m maps THREADs to API_Lemmings, this structure is used
	// to obtain language and encoding information on a per thread
	// basis.  
	NeoAPIThreadLemmingMap lemmap_m;

	static API_LEM* lem_m;

	// StringMap object - maps string keys to 
	// their position in the locale arrays
	API_StringMap sm;

	// if defaultLEM is true, we always use default messages
	bool defaultLEM;

	// set of available languages
	NeoAPILangKeySet availableLanguages_m;

	// map of encoding/language combos to locales
	NeoAPILangLocaleMap localeMap_m;

	// addLocale adds a locale to localeMap_m
	void addLocale(const char * lang);

	// getLanguages populates the set availableLanguages_m on startup
	void getLanguages();

	// rootDir_m is the root install dir for NeoCore XMS.  It is
	// used so we know where to find the locale properties files
	char rootDir_m[1024];

	// this bool is true if the LEM was initialized properly
	bool initialized;

	// this protects access
	static API_NeoCritical* lemLock;

};

#endif


