//  Copyright 2001-2002, NeoCore Inc.
//  All Rights Reserved

#ifndef NEO_CONNECTION_POOL_H
#define NEO_CONNECTION_POOL_H

/*
 * NeoConnectionPool.h
 *
 * Created on November 14, 2001
 * Based on the Java api NeoConnectionPool class
 */

#include "API_LEM.h"
#include "NeoException.h"
#include "NeoConnection.h"
#include "API_C_dll.h"
#include "NeoAPICriticalSection.h"
#include <time.h>

const int INCREMENT_COUNT = 10;

/**
 * @memo <DL><DD>This struct helps the checkedInConnection list keep track of how old
 * connections are.</DD></DL>
 * @doc <br/>This struct helps the checkedInConnection list keep track of how old
 * connections are.
 */
struct NeoConnectionTimeStamp {
	NeoConnection* connect;
	time_t time;
};

/**
 * @memo <DL><DD>This class holds a pool of NeoConnections and performs connection pool managment.</DD></DL>
 * @doc <br/>This class holds a pool of NeoConnections.
 * Applications that use several connections simultaneously can use this class.
 * Warning:
 * Developers using this class need to take care to use it properly.
 * All transactions are tied to a Session ID.  Transactions that use one SID must not
 * overlap in time.  Additionally, every time a call is made to the server using
 * a SID, that SID must not be used again until that call is finished.  Also if
 * an application uses the startTransaction endTransaction methods, these must be
 * tied to the same SID to work properly.
 *
 */
class HTTPCLIENT4_API NeoConnectionPool 
{
    private:
	
	NeoConnection** checkedOutConnections ;
	NeoConnectionTimeStamp* checkedInConnections ;
    	int totalConnections, maxConnections, timeOut, portNumber, totalCheckedOut, totalTimeOut;
	// these are to handle the indexes into the checkedInConnections
	int checkedInBegin, checkedInEnd;
    	char* hostAddress;    

	API_LEM* lem;
    
	// this protects access
	API_NeoCritical* poolLock;

 	/**
	 * @memo <DL><DD>Creates and inserts new connections created by creator.</DD></DL>
	 * @doc <br/>Creates and inserts new connections created by creator.
	 *
     */
    void createAndAddConnection()
	    throw ( NeoConnectionPoolException, NeoPoolClosedException, NeoMalformedURLException, 
	    	
                    NeoCommunicationException, NeoInvalidArgumentException );
    
    /**
	 * @memo <DL><DD>Creates a specified number of new connections and adds them to the pool.</DD></DL>
	 * @doc <br/>Creates a specified number of new connections and adds them to the pool.
	 *
	 * @param connectionCount The number of connections.
     */
    void createAndAddConnections(int connectionCount)
		throw ( NeoConnectionPoolException, NeoPoolClosedException, NeoMalformedURLException,
                        NeoCommunicationException, NeoInvalidArgumentException );
    
    /**
	 * @memo <DL><DD>Eliminates all timedOut connections from the front of the checkedIn vector.</DD></DL>
	 * @doc <br/>Eliminates all timedOut connections from the front of the checkedIn vector.
     */
    void checkTimeOuts();
    /**
	 * @memo <DL><DD>Gets the number of connections this pool could hold, but have timedout.</DD></DL>
	 * @doc <br/>Gets the number of connections this pool could hold, but have timedout.
	 * This method does not use the pool lock.
	 *
     * @return the number of timed-out connections that have been removed from the connection pool.
     */
    int getTimeOutsInternal();

    /**
	 * @memo <DL><DD>Gets the total number of NeoConnection currently idle in the pool.</DD></DL>
	 * @doc <br/>Gets the total number of NeoConnection currently idle in the pool.
	 * This method does not use the pool lock.
     * @return the number of connections that are currently idle in the connection pool.
     */
    int getIdleConnectionsInternal();




	
	public:
    /**
	 * @memo <DL><DD>Creates new a NeoConnectionPool.</DD></DL>
	 * @doc <br/>Creates new NeoConnectionPool.  
     * The maxConnections and totalConnections must be more than 1, totalConnections 
	 * must be less than maxConnections. 
     * 
     * @param totalConnections the number of connections that this pool will maintain
     * @param maxConnections the most connections that this pool will expand to
     * @param timeOut the time in seconds that a connection can be idle in the pool before 
	 * it is automatically destroyed
     * @param hostAddress the internet address of the NeoCore server
     * @param portNumber the port number used to connect to the NeoCore server
     * @exception NeoInvalidArgumentException If any parameters are invalid. 
     */
    NeoConnectionPool(int totalConnections,
		int maxConnections,
		int timeOut,
		char* hostAddress,
		int portNumber) 
		throw ( NeoConnectionPoolException, NeoMalformedURLException, NeoCommunicationException, NeoInvalidArgumentException );
    

	/**
	 * @memo <DL><DD>Creates new a NeoConnectionPool.</DD></DL>
	 * @doc <br/>Creates new NeoConnectionPool.  
     * The maxConnections and totalConnections must be more than 1, totalConnections 
	 * must be less than maxConnections. 
     * 
     * @param totalConnections the number of connections that this pool will maintain
     * @param maxConnections the most connections that this pool will expand to
     * @param timeOut the time in seconds that a connection can be idle in the pool before 
	 * it is automatically destroyed
     * @param hostAddress the internet address of the NeoCore server
     * @param portNumber the port number used to connect to the NeoCore server
	 * @param language The desired language expected from the server.
     * @param charset The desired character charset expected from the server.
	 * @param rootdir The desired location of the resource files for externalized strings.  Setting this value to NULL will cause all externalized strings to be returned as English (DEFAULT).
     * @exception NeoInvalidArgumentException If any parameters are invalid. 
     */
    NeoConnectionPool(int totalConnections,
		int maxConnections,
		int timeOut,
		char* hostAddress,
		int portNumber,
		const char *language, 
		const char *charset, 
		const char *rootdir = NULL) 
		throw ( NeoConnectionPoolException, NeoMalformedURLException, NeoCommunicationException, NeoInvalidArgumentException );
    

	/**
	 * @memo <DL><DD>Destroys a connection pool.</DD></DL>
	 * @doc <br/>Destroys a connection pool.
	 * This will destroy all internal system resources associated with this pool.
	 * This will also destroy all checked in connections.
	 * THIS WILL NOT DELETE CHECKEDOUT CONNECTIONS. 
	 *
	 */
	~NeoConnectionPool();

    /**
	 * @memo <DL><DD>Returns a connection to the pool.</DD></DL>
	 * @doc <br/>Returns a connection to the pool.
	 *
     * @param connection the NeoConnection you want to return to this pool.
     */
    void returnConnection(NeoConnection* connection)
		throw ( NeoConnectionPoolException );
    
    /**
	 * @memo <DL><DD>Gets a connection from the pool.</DD></DL>
	 * @doc <br/>Gets a connection from the pool.
	 *
     * @return A NeoConnection from the pool, or NULL if none where available
     * @exception NeoPoolClosedException If the pool has already been closed.
     */
    NeoConnection* getConnection() 
		throw ( NeoPoolClosedException );
    
    /**
	 * @memo <DL><DD>Returns the connection timeout in seconds.</DD></DL>
	 * @doc <br/>Returns the connection timeout in seconds.
	 *
     * @return the length of time a NeoConnection is allowed to be idle before it is destroyed
     */
    int getConnectionTimeOut() { return timeOut; }
    
    /**
	 * @memo <DL><DD>Sets the connection time out.</DD></DL>
	 * @doc <br/>Sets the connection time out.
	 *
     * @param timeOut The amount of time a NeoConnection is allowed to be idle before it is destroyed in seconds.
	 * @exception NeoInvalidArgumentException If the new Time-Out is less than 1
     */
    void setConnectionTimeOut(int newTimeOut)
		throw ( NeoInvalidArgumentException );

    /**
	 * @memo <DL><DD>Gets the total number of NeoConnections currently idle in the pool.</DD></DL>
	 * @doc <br/>Gets the total number of NeoConnections currently idle in the pool.
	 *
     * @return the number of connections that are currently idle in the connection pool.
     */
    int getIdleConnections();
    
    /**
   	 * @memo <DL><DD>Gets the number of connections this pool could hold, but have timedout.</DD></DL>
	 * @doc <br/>Gets the number of connections this pool could hold, but have timedout.
	 *
     * @return the number of timed-out NeoConnections that have been removed from the connection pool.
     */
    int getTimeOuts();
    
	/**
	 * @memo <DL><DD>Removes connectionCount idle NeoConnections from the connection pool.</DD></DL>
	 * @doc <br/>Removes connectionCount idle NeoConnections from the connection pool.
	 * If connectionsCount is larger than the number of idle connections then all idle connections are effectively closed.
	 *
	 * @param connectionCount the number of NeoConnections to be removed from the idle list
	 * @exception NeoInvalidArgumentException If the connection count is less than 1;
	 */
	void removeConnections(int connectionCount)
		throw ( NeoInvalidArgumentException );
    
    /**
	 * @memo <DL><DD>Removes all connections from the idle pool and closes this pool to new requests.</DD></DL>
	 * @doc <br/>Removes all connections from the idle pool and closes this pool to new requests.
     * Once all connnection have been removed this method blocks all requestors from obtaining connnections.
	 *
     * @exception ConnnectionPoolClosed If requestors attempt to obtain a connection after this method has been called.
     */
    void removeAllConnections();
    
    /**
	 * @memo <DL><DD>Returns the number of NeoConnections that are currently active in the connection pool.</DD></DL>
	 * @doc <br/>Returns the number of NeoConnections that are currently active in the connection pool.
	 *
     * @return The number of NeoConnections that are currently active in the connection pool.
     */
    int getActiveConnections();
   
    /**
	 * @memo <DL><DD>Resets all of the time stamps on all NeoConnections.</DD></DL>
	 * @doc <br/>Resets all of the time stamps on all NeoConnections.
	 *
     */
    void resetTimeoutMonitor();
    
    /**
	 * @memo <DL><DD>Refills the pool with valid NeoConnections.</DD></DL>
	 * @doc <br/>Refills the pool with valid NeoConnections.
     * This will remove all timed-out NeoConnections.
     * Then it will add new NeoConnections for all timed-out connections.
	 * 
     * @exception NeoPoolClosedException - If the Connection Pool is closed
     */
    void replenishConnections()
		throw  ( NeoPoolClosedException );
    
    /**
	 * @memo <DL><DD>Increases the active connection count by connectionCount.</DD></DL>
	 * @doc <br/>Increases the active connection count by connectionCount.
     * If this new count is more than totalConnections, totalConnections will be increased.
     * If connectionCount is less than the number of time outs, totalConnections will not be changed.
     *
     * @param connectionCount the number of connections to be added
     * @exception NeoPoolClosedException - If the Connection Pool is closed
     * @exception NeoConnectionPoolException - If resulting number of connections would be greater than the max.
	 * @exception NeoInvalidArgumentException If the connection count is less than 1;
     */
    void addConnections(int connectionCount)
		throw ( NeoConnectionPoolException, NeoPoolClosedException, NeoInvalidArgumentException);
    
    
    char* toChars() ;
    
};
#endif //NEO_CONNECTION_POOL_H

