#ifndef CURSOR_H_INCLUDED_C3CB739E
#define CURSOR_H_INCLUDED_C3CB739E

#include "NeoResultChunk.h"
#include "NeoResultProfile.h"
#include "NeoException.h"

#ifdef WIN32
#pragma warning( disable : 4290 ) 
#endif



 /** 
   * NeoCursor allows the client to manuever through a NeoQueryResult and obtain the
   * NeoCursor's XML at the target axis query element level.
   *
   * @author Richard Monteleone
   * @version 1.0
   *
   * @see NeoResultProfile
   * @see NeoQueryResult
   */

class  HTTPCLIENT4_API NeoCursor
{
  public:
    /**
      * Constructor for a NeoCursor.  Points the NeoCursor to  the initialResultChunkID at the 
	  * initialStartingPosition.
      *
	  * @param resultChunks The vector of NeoResultChunks that the cursor will cursor over
	  * @param initialResultChunk The NeoResultChunk ID where this NeoCursor should be positioned
	  * @param initialStartingPostion The relative starting position in the initialResultChunk to place
	  * this NeoCursor
	  * @param numAxisElements The number of Axis Elements that the parent NeoQueryResult can hold when
	  * fully loaded.
	  * @param resultProfile The result profile that was used when the queryXML was executed
	  *
      * @excepton NeoInvalidArgumentException - If any of the parameters are invalid
      * @exception NeoClientException - Unhandled client-side error condition has occured. 
	  */
    NeoCursor( const vector <NeoResultChunk*>*  resultChunks, const int initialResultChunkID, 
			   const int initialStartingPosition, const __int64 numAxisElements,
		       const NeoResultProfile* const resultProfile) 
	throw (NeoInvalidArgumentException,
		   NeoClientException);




 

   /** Retrieves the XML for the current cursor position and the next or previous additional cursor positions. The
	  * NeoCursor remains in its original position. Also, if the NeoResultProfile#resultChunkTransferMode was set 
	  * to Auto (i.e. 1) and the XML requested ventures into another NeoResultChunk that is in the state of 
	  * NEVER_LOADED this method will automatically load the NeoResultChunk. 
      * When finished with the returned XML (char*) call releaseXML(char*) to free the XML. 	
	  *	 
      * @param additionalPositions if positive the current cursor position and the next N cursor positon's XML are
	  * returned, if negative the current cursor position and the previous N cursor position's XML are returned
	  * if zero the current cursor position's XML are returned
	  *
      * @excepiton NeoInvalidSessionException - If the session that was used for the initial query that constructed 
	  * this NeoQueryResult is no longer valid
      * @exception NeoManualLoadRequiredException - If the NeoCursor is positioned in a NeoResultChunk that needs loading 
	  * and you are operating in manual-mode. You can obtain the NeoResultChunk's ID that needs to be loaded from the exception. 
      * @exception NeoDestroyedResultChunkException - If the NeoCursor is positioned in a NeoResultChunk that has been destroyed
      * @exception NeoCursorOutOfRangeException - If an attempt is made to obtain XML for a cursor positon that does not
      * exist within this NeoQueryResult. You can obtain the boudary NeoCursor position from the exception
      * @exception NeoCommunicationException - If there was a problem communicating with the NeoCore XMS Server to 
	  * load (i.e. auto-mode enabled) a NeoResultChunk
      * @exception NeoClientException - Unhandled client error condition has occured
      * @exception NeoServerException - If there was an internal NeoCore XMS error.
	  *
	  * @see NeoResultProfile
	  * @see NeoResultChunk
	  * @see NeoCursor
	  * @see releaseXML
	  *
 	  * @return The XML that the NeoCursor is currently pointing to and any additionalPositons, example XML:
      *	<Query-Results>
	  *    <YourData>1</YourData>
	  *    <YourData>1</YourData>
	  * </Query-Results>
	  * 
	  */
    char* get_XML(const int additionalPositions = 0) 
	throw ( NeoInvalidSessionException,
		    NeoManualLoadRequiredException,
		    NeoDestroyedResultChunkException,
		    NeoCursorOutOfRangeException,
		    NeoCommunicationException,
		    NeoClientException,
			NeoServerException);


    /**
      * Checks to see if the cursor is currently placed within the NeoResultChunk
      * identified by resultChunkID
      * 
	  * @param resultChunkID The ID of the NeoResultChunk that you are destroying 
      * 
	  * @return true if this NeoCursor is in the result chunk, false otherwise
	  */
    const bool isInNeoResultChunk(const int resultChunkID) const;


    /**
	  * Determines if this is a valid cursor position,  the state of the NeoResultChunk 
	  * where the cursorPosition resides is not factored into whether or not it is valid.
	  *
	  * @param cursorPosition the absolute cursor positon
	  *
	  * @see isUsablePositon
	  *
	  * @return true if the cursorPostion is valid for this NeoQueryResult, false otherwise.
	  */
    const bool isValidPosition(const __int64 cursorPosition) const;



    /**
	  * Determines if the cursorPosition is usable, usable means that the the 
	  * cursorPosition exists and the state of NeoResultChunk that contains
	  * the cursorPositon has not been DESTROYED, must be either LOADED or NEVER_LOADED.
      * 
	  * @see NeoResultChunk#NeoResultChunkState
	  *
	  * @return true if it is usable, otherwise false
	  */
    const bool isUsablePosition(const __int64 cursorPosition) const;



    /** Moves this NeoCursor forward or backward within the NeoQueryResult. Additionally, if the
	  * NeoResultProfile#resultChunkDisacardMode was set to Auto (i.e. 1) and this NeoCursor leaves a
	  * NeoResultChunk that NeoResultChunk will be automatically destroyed. Also, if the
	  * NeoResultProfile#resultChunkTransferMode was set to Auto (i.e. 1) and this NeoCursor enters 
	  * into a NeoResultChunk that is in the state of NEVER_LOADED this method will automatically
	  * load the NeoResultChunk. 
	  *
	  * @param movement provide a postitive number to move it forward, a negative
      * number to move it backward in the NeoQueryResult
	  *
	  * @exception NeoInvalidArgument - If movement is equal to zero
      * @exception NeoInvalidSessionException - If the session that was used for the initial query that constructed 
	  * this NeoQueryResult is no longer valid
      * @exception NeoManualLoadRequiredException - If the NeoCursor is positioned in a NeoResultChunk that needs loading 
	  * and you are operating in manual-mode. You can obtain the NeoResultChunk's ID that needs to be loaded from the exception. 
      * @exception NeoDestroyedResultChunkException - If the NeoCursor is positioned in a NeoResultChunk that has been destroyed
      * @exception NeoCursorOutOfRangeException - If an attempt is made to move to a cursor position that does not
      * exist within this NeoQueryResult. You can obtain the boundary NeoCursor position from the exception
      * @exception NeoCommunicationException - If there was a problem communicating with the NeoCore XMS Server to 
	  * load (i.e. auto-mode enabled) a NeoResultChunk
      * @exception NeoClientException - Unhandled client error condition has occured
      * @exception NeoServerException - If there was an internal NeoCore XMS error.
	  *
	  * @see NeoResultProfile
	  * @see NeoResultChunk
	  * @see NeoResultChunk#NeoResultChunkState
	  * @see NeoQueryResult
	  *
	  * @return The current cursor position after the movement takes place
	  */ 
    const __int64 move(const __int64 movement = 1)
	throw ( NeoInvalidArgumentException,
			NeoInvalidSessionException,
			NeoManualLoadRequiredException,
			NeoDestroyedResultChunkException,
			NeoCursorOutOfRangeException,
			NeoCommunicationException,
			NeoClientException,
			NeoServerException);



	/** Repositions this NeoCursor within the NeoQueryResult. Additionally, if the
	  * NeoResultProfile#resultChunkDisacardMode was set to Auto (i.e. 1) and this NeoCursor leaves a
	  * NeoResultChunk that NeoResultChunk will be automatically destroyed. Also, if the
	  * NeoResultProfile#resultChunkTransferMode was set to Auto (i.e. 1) and this NeoCursor enters 
	  * into a NeoResultChunk that is in the state of NEVER_LOADED this method will automatically
	  * load the NeoResultChunk. 
	  *
	  * @param newCursorLocation the absolute location where you want to place the NeoCursor in the NeoQueryResult, the lower
	  * boundary of a NeoQueryResult is always zero. 
	  *
      * @exception NeoInvalidSessionException - If the session that was used for the initial query that constructed 
	  * this NeoQueryResult is no longer valid
      * @exception NeoManualLoadRequiredException - If the NeoCursor is positioned in a NeoResultChunk that needs loading 
	  * and you are operating in manual-mode. You can obtain the NeoResultChunk's ID that needs to be loaded from the exception. 
      * @exception NeoDestroyedResultChunkException - If the NeoCursor is positioned in a NeoResultChunk that has been destroyed
      * @exception NeoCursorOutOfRangeException - If an attempt is made to reposition to a cursor position that does not
      * exist within this NeoQueryResult. You can obtain the boundary NeoCursor position from the exception
      * @exception NeoCommunicationException - If there was a problem communicating with the NeoCore XMS Server to 
	  * load (i.e. auto-mode enabled) a NeoResultChunk
      * @exception NeoClientException - Unhandled client error condition has occured
      * @exception NeoServerException - If there was an internal NeoCore XMS error.
	  * 
	  * @see NeoResultProfile
	  * @see NeoResultChunk
	  * @see NeoResultChunk#NeoResultChunkState
	  * @see NeoQueryResult
	  *
	  * @return The current cursor position after the reposition takes place
	  */
    const __int64 reposition(const __int64 newCursorLocation)
	throw ( NeoInvalidSessionException,
			NeoManualLoadRequiredException,
			NeoDestroyedResultChunkException,
			NeoCursorOutOfRangeException,
			NeoCommunicationException,
			NeoClientException,
			NeoServerException);

	/**
	  * Destructor for this NeoCursor. 
	  *
	  */
    virtual ~NeoCursor();



    /**
	  * Retrieves the current position of this NeoCursor in the NeoResultChunks
	  *
	  * @see NeoResultProfile
	  * @see NeoResultChunk
	  *
      * return the current position of the NeoCursor in this NeoQueryResult
	  */
    const __int64 get_Position() const;



    /** Retrieves the ID of the NeoResultChunk where this NeoCursor is currently positioned
	  *
	  * @see NeoResultProfile
	  * @see NeoResultChunk
	  *
      * @return the ID of the NeoResultChunk
	  */
    const int get_CurrentNeoResultChunk() const;



  private:

    /**
      * Takes the unwrappedXML and wraps it in the TAG_WRAPPER_START and 
	  * TAG_WRAPPER_END so that it valid XML, the wrapped XML is returned. The original
	  * unwrappedXML must be freed by the caller of this method. The returned wrapped
      * XML must also be freed by the requestor when they are finished with it.
	  */
	char*  wrapXML(const char* const unwrappedXML) 
	throw ( NeoClientException );


	void convertAbsToRelCursorPos(const __int64 absoluteCursorPosition,
								  int &neoChunkID, int &relPosInNeoChunk) const;
	/**
	  * @return the conversion of the relative cursor position to an absolute position
	  */
	const __int64 convertRelToAbsCursorPos(const int neoChunkID, const int relativeCursorPos) const;
		

    /**
	  * The current position of the NeoCursor in its current NeoResultChunk
	  */
    int m_relativeChunkPosition;



	/**
	  * The ID of the NeoResultChunk that we are currently positioned in. 
	  */ 
    int  m_cursorResultChunk;



	/** 
	  * The result chunks, as we move the NeoCursor around they may ask for XML, chunk state,
      * chunk size, etc..)
	  */
	const vector <NeoResultChunk*>*  m_theResultChunks; 



	/** 
	  * How many axis elements can the parent NeoQueryResult hold.
	  */ 
	__int64 m_numAxisElements;



	/**
	  * Did they have a transfer mode of 1, auto-load chunks that are in the NEVER_LOADED
	  * state automatically
	  */
	bool m_autoLoadEnabled;



	/**
	  * Did they have the ChunkDiscardMode set to 1, auto-discard result chunks as this
	  * NeoCursor leaves the scope of a NeoResultChunk
	  */
	bool m_autoDiscardEnabled;

	
	// When constructing XML we sometimes move the cursor, in this case we do not want to destroy
	// NeoResultChunks. This is an override that controls whether or not we cleanup NeoResultChunks when repositioning the
	// NeoCursor. 
	bool m_IgnoreCleanUP;

};




#endif /* CURSOR_H_INCLUDED_C3CB739E */

