#ifndef NEO_QUERYRESULT_H
#define NEO_QUERYRESULT_H


#include "API_C_dll.h"
#include "NeoAPICriticalSection.h"
#include "NeoException.h"
#include "UrlConnection.h"
#include "NeoResultProfile.h"
#include "NeoResultChunk.h"
#include "NeoCursor.h"



#include <vector>
using namespace std;


#ifdef WIN32
#pragma warning( disable : 4290 ) 
#endif



#ifdef DEBUG
#	define FAIL_LOC(class)	class:[__LINE__ ],
#else
#	define FAIL_LOC(class)	class,
#endif



// XML Tag Definitions all here in case we need to change them
const char TAG_TRUNCATED[]		 =	"Truncated";
const char TAG_CHUNKXML[]		 =	"ChunkXML";
const char TAG_AXISELEMENTS[]	 = 	"AxisElements";
const char TAG_SESSIONID[]		 = 	"SessionID";
const char TAG_RESULTCHUNK[]	 =	"NeoResultChunk";
const char TAG_ID[]				 =	"ID";
const char TAG_OFFSET[]			 = 	"Offset";
const char TAG_AE[]				 = 	"AE";
const char TAG_CURSORLOC[]		 =	"NeoCursorLoc";
const char TAG_NUMRESULTCHUNKS[] = 	"NeoResultChunks";
const char TAG_WRAPPER_START[]   = 	"<Query-Results>";
const char TAG_WRAPPER_END[]	 = 	"\n</Query-Results>";
const char TAG_UML_SYNTAX[]		 =	"<></>";

// XML Declaration start and end; insert charset between to make full decl
const char XML_DECL_START[]      = "<?xml version=\"1.0\" encoding=\"";
const char XML_DECL_END[]        = "\" ?>\n";



// Commands that are sent to the NeoCore XMS Server for Chunking/Cursoring support. 
const char CMD_QUERY_CHUNKED[]         = "QUERY_CHUNKED";
const char CMD_QUERYUPDATE_CHUNKED[]   = "QUERYUPDATE_CHUNKED";
const char CMD_SERVE_NEOCHUNK[]	       = "SERVE_NEOCHUNK";
const char CMD_DESTROY_NEOCHUNK[]      = "DESTROY_NEOCHUNK";
const char CMD_DESTROY_NEORESULT[]     = "DESTROY_NEORESULT";
const char CMD_ISACTIVE_NEORESULT[]    = "ISACTIVE_NEORESULT";




/** 
  * @memo <DL><DD>NeoQueryResult is the  client-side class that provides the capability to support
  * chunking and cursoring of a NeoCore XMS Server query result. </DD></DL>
  * @doc <br/>NeoQueryResult is the  client-side class that provides the capability to support
  * chunking and cursoring of a NeoCore XMS Server query result. 
  * This class is thread-safe.
  *
  * @author Richard A. Monteleone
  *
  * @version 1.0
  *
  * @see NeoConnection#queryXML
  * @see NeoConnection#queryXMLUpdateIntent
  * @see SessionManagedNeoConnection#queryXML
  * @see SessionManagedNeoConnection#queryXMLUpdateIntent
  * @see NeoResultProfile
  * @see NeoResultChunk
  * @see NeoCursor
  */


class HTTPCLIENT4_API NeoQueryResult
{

  public:


	/**
	  * @memo <DL><DD>Destructor, destroys all the NeoResultChunks and their AxisElements.</DD></DL>
      * @doc <br/>Destructor, destroys all the NeoResultChunks and their AxisElements.  Communicates with
      * the NeoCore XMS Server to destroy any NeoResultChunks that have not been transferred
      * to this client.
	  *
	  */
    virtual ~NeoQueryResult();



	/**
	  * @memo <DL><DD>Destroys a NeoResultChunk, primary use is when the client elects to manually manage the loading and 
	  * destroying of NeoResultChunks as specified in the NeoResultProfile.</DD></DL>
      * @doc <br/>Destroys a NeoResultChunk, primary use is when the client elects to manually manage the loading and 
	  * destroying of NeoResultChunks as specified in the NeoResultProfile. If the NeoResultChunk identified by
	  * resultChunkID has never been tranferred to the client (i.e NEVER_LOADED state) then it will be destroyed
	  * on the NeoCore XMS Server. Once destroyed, the contents for a NeoResultChunk can not be re-obtained. 
      *  
	  * @param resultChunkID The ID of the NeoResultChunk that you are destroying 
	  *
      * @exception NeoInvalidArgumentException - If the resultChunkID does not identify a valid NeoResultChunk
      * @exception NeoInvalidSessionException - If the session that was used for the initial query that constructed 
	  * this NeoQueryResult is no longer valid
      * @exception NeoCommunicationException - If there was a problem communicating with the NeoCore XMS Server
      * @exception NeoClientException - Unhandled client error condition has occured
      * @exception NeoServerException - If there was an internal NeoCore XMS error.
	  *
	  * @see NeoResultProfile#
	  * @see NeoResultChunk#NeoResultChunkState
	  * @see NeoResultChunk#get_state
	  * 
	  * @return true if the NeoResultChunk was LOADED or NEVER_LOADED, false otherwise (NeoResultChunk has a state of DESTROYED)
	  */
	bool destroyNeoResultChunk(const int resultChunkID) 
	throw ( NeoInvalidArgumentException,
			NeoInvalidSessionException,
			NeoCommunicationException,
			NeoClientException,
			NeoServerException);	



	/**
	  * @memo <DL><DD>Returns true if the NeoQueryResult was truncated.</DD></DL>
      * @doc <br/>Was this NeoQueryResult truncated by the NeoCore XMS Server?
	  * 
	  * @see NeoResultProfile
	  *
	  * @return true if this NeoQueryResult has been truncated, false otherwise.
	  */
    const bool isTruncated() const;



	/**
	  * @memo <DL><DD>Returns true if the result chunk ID is valid.</DD></DL>
      * @doc <br/>Does this resultChunkID exist for this NeoQueryResult?  A valid ID simply
	  * means that the ID exists, it does not consider the state of the NeoResultChunk.
      *
	  * @param the ID of the NeoResultChunk 
      *
	  * @see NeoQueryResult#get_NeoResultChunkState
	  *
      * @return true if the resultChunkID exists, false otherwise 
	  */
    const bool isValidNeoResultChunk(const int resultChunkID) const;	


    /** 
	  * @memo <DL><DD>Returns true if the cursor location is valid in any result chunk.</DD></DL>
      * @doc <br/>Does this cursorPosition exist in this NeoQueryResult? A valid cursorPosition simply
	  * means that the position exists, it does not consider the state of the NeoResultChunk
	  * that holds this cursor position. 
	  *
	  * @param cursorPosition The cursor position.
	  *
	  * @see NeoResultChunk#NeoResultChunkState
	  *
      * @return true if the cursorPostion exists, false otherwise. 
	  */
    const bool isValidNeoCursorPosition(const int cursorPosition) const;

	/** 
	  * @memo <DL><DD></DD></DL>Determines if the cursorPosition is usable, usable means that the cursorPosition exists and the 
	  * NeoResultChunk that holds the cursorPositon is not in a state of DESTROYED. (i.e. LOADED or NEVER_LOADED)
      * @doc <br/>Determines if the cursorPosition is usable, usable means that the cursorPosition exists and the 
	  * NeoResultChunk that holds the cursorPositon is not in a state of DESTROYED. (i.e. LOADED or NEVER_LOADED) 
	  *
	  * @param cursorPosition The cursor position.
	  *
	  * @see NeoResultChunk#NeoResultChunkState
	  *
      * @return true if the cursorPosition is usable, false otherwise. 
	  */
    const bool isUsableNeoCursorPosition(const __int64 cursorPosition) const;


	/** 
	  * @memo <DL><DD>Retrieves the current state of the NeoResultChunk as identified by resultChunkID.</DD></DL>
      * @doc <br/>Retrieves the current state of the NeoResultChunk as identified by resultChunkID.
	  *
	  * @param resultChunkID The ID of the NeoResultChunk.
	  *
	  * @exception  NeoInvalidArgumentException - If the resultChunkID does not identify a valid NeoResultChunk.
	  *
	  * @see NeoResultChunk#NeoResultChunkState
	  * @see NeoResultChunk#get_state
	  *
      * @return the State of the NeoResultChunk. States are as follows: <br/>
      *  <UL><LI><B>-1 = (DESTROYED)</B> Previously Loaded and deleted (XML for this chunk of 
      *        Axis Elements no longer exists on the client or server).</LI>
      *   <LI><B>0 =  (NEVER_LOADED)</B> Never Loaded with XML from the NeoCore XMS Server (XML exists 
      *        on the server but has never been transferred to the client. </LI>
      *   <LI><B>1 =  (LOADED)</B> Loaded with XML from the NeoCore XMS Server (XML for this NeoResultChunk no
      *  	    longer exists on the server).</LI></UL>
      *
	  */
	const int get_NeoResultChunkState(const int resultChunkID) const 
	throw (NeoInvalidArgumentException);	

	
    /**
	  * @memo <DL><DD>Loads the NeoResultChunk as identified by resultChunkID.</DD></DL>
      * @doc <br/>Loads the NeoResultChunk as identified by resultChunkID. This is typically used when elect to manually manage
	  * the NeoResultChunks in your NeoResultProfile
	  *
	  * @param resultChunkID The ID of the NeoResultChunk that you are loading.
	  *
      * @exception NeoInvalidArgumentException - If the resultChunkID does not identify a valid NeoResultChunk.
      * @exception NeoInvalidSessionException - If the session that was used for the initial query that constructed.
	  * this NeoQueryResult is no longer valid.
	  * @exception NeoDestroyedResultChunkException - If this NeoResultChunk was previously loaded and destroyed.
      * @exception NeoCommunicationException - If there was a problem communicating with the NeoCore XMS Server.
      * @exception NeoClientException - Unhandled client error condition has occured.
      * @exception NeoServerException - If there was an internal NeoCore XMS error.
	  *
	  * @see NeoResultProfile
	  * @see NeoResultChunk#NeoResultChunkState
	  * @see NeoResultChunk#get_state
	  *
	  * @return true if the NeoResultChunk was loaded, false otherwise (NeoResultChunk already has a state of LOADED).
	  */
    const bool loadNeoResultChunk(const int resultChunkID)
    throw ( NeoInvalidArgumentException,
		    NeoInvalidSessionException,
		    NeoDestroyedResultChunkException,
		    NeoCommunicationException,
		    NeoClientException,
		    NeoServerException);



    /** 
	  * @memo <DL><DD>Retrieves the current Statistics in XML for this NeoQueryResult. </DD></DL>
      * @doc <br/>Retrieves the current Statistics in XML for this NeoQueryResult. 
      * When finished with the returned XML (char*) call releaseXML(char*) to free the XML. 	
	  *
      * @exception NeoClientException - Unhandled client error condition has occured.
	  *
	  * @see NeoResultProfile
	  * @see NeoResultChunk
	  * @see NeoCursor
	  * @see releaseXML
	  *
      * @return the current Statistics, example XML:<br/><br/>
      *
      *	&lt;Query-Results&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
      *		&lt;ID&gt;0&lt;/ID&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
      *		&lt;Truncated&gt;1&lt;/Truncated&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
      *		&lt;SessionID&gt;12345678&lt;/SessionID&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
      *		&lt;TotalNeoResultChunks&gt;3&lt;/TotalNeoResultChunks&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
      *		&lt;TotalAxisElements&gt;2500&lt;/TotalAxisElements&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
      *		&lt;LoadedNeoResultChunks&gt;1&lt;/LoadedNeoResultChunks&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
      *		&lt;LoadedAxisElements&gt;1000&lt;/LoadedAxisElements&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
      *		&lt;NeoResultChunk&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
      *			&lt;ID&gt;0&lt;/ID&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
      *			&lt;Offset&gt;0&lt;/Offset&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
      *			&lt;AxisElements&gt;1000&lt;/AxisElements&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
      *         &lt;State&gt;LOADED&lt;/State&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
      *         &lt;NeoCursorLoc&gt;999&lt;/NeoCursorLoc&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
      *     &lt;/NeoResultChunk&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
      *     &lt;NeoResultChunk&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
      *			&lt;ID&gt;1&lt;/ID&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
      *		   	&lt;Offset&gt;1000&lt;/Offset&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
      *			&lt;AxisElements&gt;1000&lt;/AxisElements&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
      *         &lt;State&gt;DESTROYED&lt;/State&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
      *		&lt;/NeoResultChunk&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
      *     &lt;NeoResultChunk&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
      *			&lt;ID&gt;2&lt;/ID&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
      *			&lt;Offset&gt;2000&lt;/Offset&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
      *			&lt;AxisElements&gt;500&lt;/AxisElements&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
      *         &lt;State&gt;NEVER_LOADED&lt;/State&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
      *		&lt;/NeoResultChunk&gt;<br/>
      *	&lt;/Query-Results&gt;<br/><br/>
      *     
      *
	  */
	char* get_StatisticsXML() const throw (NeoClientException);

	

    /** 
	  * @memo <DL><DD>Retrieves the ID of the NeoResultChunk where the NeoCursor is currently positioned.</DD></DL>
      * @doc <br/>Retrieves the ID of the NeoResultChunk where the NeoCursor is currently positioned.
	  *
	  * @see NeoResultProfile
	  * @see NeoResultChunk
	  * @see NeoCursor
	  *
      * @return The ID of the NeoResultChunk
	  */
    const int get_NeoCursorsNeoResultChunk() const;



    /** 
	  * @memo <DL><DD>Retrieves the XML contained by the NeoResultChunk as identified by resultChunkID.</DD></DL>
      * @doc <br/>Retrieves the XML contained by the NeoResultChunk as identified by resultChunkID. Also, if the 
	  * NeoResultProfile#resultChunkTransferMode was set to Auto (i.e. 1) and the XML requested resides in a
	  * NeoResultChunk that is in the state of NEVER_LOADED this method will automatically load the 
	  * NeoResultChunk. <br/><br/>
	  *
	  * Note: NeoResultChunk's are never destroyed when using this method, even if the 
	  * NeoResultProfile resultChunkDiscardMode is set to AUTO, it is ignored. <br/><br/>
	  *
      * When finished with the returned XML (char*) call releaseXML(char*) to free the XML. 
	  *
	  * @param resultChunkID The ID of the NeoResultChunk that you requesting XML for.
      * 
      * @exception NeoInvalidArgumentException - If the resultChunkID does not identify a valid NeoResultChunk.
      * @exception NeoManualLoadRequiredException - If the data for this NeoResultChunk has not been loaded .
      * @exception NeoInvalidSessionException - If the session that was used for the initial query that constructed
	  * this NeoQueryResult is no longer valid.
      * @exception NeoDestroyedResultChunkException - If the NeoResultChunk was previously loaded and destroyed.
      * @exception NeoCommunicationException - If there was a problem communicating with the server to load
	  * (auto-mode) the NeoResultChunk.
      * @exception NeoClientException - Unhandled client error condition has occured.
      * @exception NeoServerException - If there was an internal NeoCore XMS error.
	  *
	  * @see NeoResultProfile
	  * @see NeoResultChunk
	  * @see NeoResultChunk#NeoResultChunkState
	  * @see releaseXML
	  *
	  * @return The XML contained by the NeoResultChunk, example XML:<br/><br/>
	  *
      *	&lt;Query-Results&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
	  *    &lt;YourData&gt;1&lt;/YourData&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
	  *    &lt;YourData&gt;2&lt;/YourData&gt;<br/>
	  * &lt;/Query-Results&gt;<br/>
	  *
	  */
	char* get_NeoResultChunkXML(const int resultChunkID)
    throw ( NeoInvalidArgumentException,
		    NeoInvalidSessionException,
		    NeoManualLoadRequiredException,
		    NeoDestroyedResultChunkException,
		    NeoCommunicationException,
		    NeoClientException,
		    NeoServerException);												 



    /** 
	  * @memo <DL><DD>Retrieves the XML for the current cursor position and the next or previous additional cursor positions.</DD></DL>
      * @doc <br/>Retrieves the XML for the current cursor position and the next or previous additional cursor positions. The
	  * NeoCursor remains in its original position. Also, if the NeoResultProfile#resultChunkTransferMode was set 
	  * to Auto (i.e. 1) and the XML requested ventures into another NeoResultChunk that is in the state of 
	  * NEVER_LOADED this method will automatically load the NeoResultChunk.<br/><br/>
	  *
	  * Note: NeoResultChunk's are never destroyed when using this method, even if the 
	  * NeoResultProfile resultChunkDiscardMode is set to AUTO, it is ignored. <br/><br/>
	  *
      * When finished with the returned XML (char*) call releaseXML(char*) to free the XML. 	
	  *
      * @param additionalPositions  If positive, the current cursor position and the next N cursor position's XML are
	  * returned.  If negative, the current cursor position and the previous N cursor position's XML are returned
	  * if zero the current cursor position's XML are returned. If 0 is provided, the current cursor's position XML
	  * is returned. 
	  *
      * @exception NeoInvalidSessionException  If the session that was used for the initial query that constructed 
	  * this NeoQueryResult is no longer valid
      * @exception NeoManualLoadRequiredException  If the NeoCursor is positioned in a NeoResultChunk that needs loading 
	  * and you are operating in manual-mode. You can obtain the NeoResultChunk's ID that needs to be loaded from the exception. 
      * @exception NeoDestroyedResultChunkException  If the NeoCursor is positioned in a NeoResultChunk that has been destroyed.
      * @exception NeoCursorOutOfRangeException  If an attempt is made to obtain XML for a cursor positon that does not
      * exist within this NeoQueryResult. You can obtain the boundary NeoCursor position from the exception.
      * @exception NeoCommunicationException  If there was a problem communicating with the NeoCore XMS Server to 
	  * load (i.e. auto-mode enabled) a NeoResultChunk.
      * @exception NeoClientException  Unhandled client error condition has occured.
      * @exception NeoServerException  If there was an internal NeoCore XMS error.
	  *
	  * @see NeoResultProfile
	  * @see NeoCursor
	  * @see NeoResultChunk
	  * @see NeoResultChunk NeoResultChunkState
	  * @see releaseXML
	  *
 	  * @return The XML that the NeoCursor is currently pointing to and any additonalPositons, example XML:<br/><br/>
	  *
      *	&lt;/Query-Results&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
	  *    &lt;YourData&gt;1&lt;/YourData&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
	  *    &lt;YourData&gt;2&lt;/YourData&gt;<br/>
	  * &lt;/Query-Results&gt;<br/>
	  * 
	  */
    char* get_NeoCursorXML(const int additionalPositions = 0)
	throw ( NeoInvalidSessionException,
			NeoManualLoadRequiredException,
			NeoDestroyedResultChunkException,
			NeoCursorOutOfRangeException,
			NeoCommunicationException,
			NeoClientException,
			NeoServerException);

	
    /**
	  * @memo <DL><DD>Retrieves the current position of the NeoCursor in this NeoQueryResult.</DD></DL>
      * @doc <br/>Retrieves the current position of the NeoCursor in this NeoQueryResult.
	  *
	  * @see NeoResultProfile
	  * @see NeoCursor
	  *
      * @return The current position of the NeoCursor in this NeoQueryResult
	  */
    const __int64 get_NeoCursorPosition() const;



    /**
	  * @memo <DL><DD>Retrieves the number of NeoResultChunks that this NeoQueryResult can contain
	  * if fully loaded.</DD></DL>
      * @doc <br/>Retrieves the number of NeoResultChunks that this NeoQueryResult can contain
	  * if fully loaded. 
	  *
	  * @see NeoResultProfile
	  *
      * @return The number of Axis Elements 
	  */
	const int get_TotalNeoResultChunks() const;



    /**
	  * @memo <DL><DD>Retrieves the number of Axis Elements that this NeoResultChunk can contain
	  * if fully loaded. </DD></DL>
      * @doc <br/>Retrieves the number of Axis Elements that this NeoResultChunk can contain
	  * if fully loaded. 
	  *
	  * @param resultChunkID The ID of the NeoResultChunk.
	  *
      * @exception NeoInvalidArgumentException  If the resultChunkID does not identify a valid NeoResultChunk.
	  *
	  * @see NeoResultChunk
	  *
      * @return The number of Axis Elements resultChunkID can contain
	  */
	const int get_TotalAxisElementsNeoResultChunk(const int resultChunkID) const
	throw ( NeoInvalidArgumentException);



    /**
	  * @memo <DL><DD>Retrieves the current number of NeoResultChunks that this NeoQueryResult currently has
	  * loaded.</DD></DL>
      * @doc <br/>Retrieves the current number of NeoResultChunks that this NeoQueryResult currently has
	  * loaded.
	  *
	  * @see NeoResultProfile
	  *
      * @return The current number of Axis Elements.
	  */
	const int get_LoadedNeoResultChunks() const;



    /**
	  * @memo <DL><DD>Retrieves the number of Axis Elements that this NeoQueryResult can contain
	  * if fully loaded.</DD></DL>
      * @doc <br/>Retrieves the number of Axis Elements that this NeoQueryResult can contain
	  * if fully loaded. 
	  *
	  * @see NeoResultProfile
	  *
      * @return the number of Axis Elements 
	  */
	const __int64 get_TotalAxisElements() const;



    /**
	  * @memo <DL><DD>Retrieves the current number of Axis Elements that this NeoQueryResult currently has
	  * loaded.</DD></DL>
      * @doc <br/>Retrieves the current number of Axis Elements that this NeoQueryResult currently has
	  * loaded. (i.e. Axis Elements currently loaded on the client).
	  *
	  * @see NeoResultProfile
	  *
      * @return the current number of Axis Elements 
	  */
	const __int64 get_LoadedAxisElements() const;



    /** 
	  * @memo <DL><DD>Moves the NeoCursor forward or backward within this NeoQueryResult.</DD></DL>
      * @doc <br/>Moves the NeoCursor forward or backward within this NeoQueryResult. Additionally, if the
	  * NeoResultProfile#resultChunkDisacardMode was set to Auto (i.e. 1) and the NeoCursor leaves a
	  * NeoResultChunk that NeoResultChunk will be automatically destroyed. Also, if the
	  * NeoResultProfile#resultChunkTransferMode was set to Auto (i.e. 1) and the NeoCursor enters 
	  * into a NeoResultChunk that is in the state of NEVER_LOADED this method will automatically
	  * load the NeoResultChunk. 
	  *
	  * @param movement Provide a postitive number to move it forward, a negative
      * number to move it backward in this NeoQueryResult.
	  *
	  * @exception NeoInvalidArgument  If movement is equal to zero.
      * @exception NeoInvalidSessionException  If the session that was used for the initial query that constructed 
	  * this NeoQueryResult is no longer valid.
      * @exception NeoManualLoadRequiredException  If the NeoCursor is positioned in a NeoResultChunk that needs loading 
	  * and you are operating in manual-mode.  You can obtain the NeoResultChunk's ID that needs to be loaded from the exception. 
      * @exception NeoDestroyedResultChunkException  If the NeoCursor is positioned in a NeoResultChunk that has been destroyed
      * @exception NeoCursorOutOfRangeException  If an attempt is made to move to a cursor position that does not
      * exist within this NeoQueryResult. You can obtain the boundary NeoCursor position from the exception
      * @exception NeoCommunicationException  If there was a problem communicating with the NeoCore XMS Server to 
	  * load (i.e. auto-mode enabled) a NeoResultChunk.
      * @exception NeoClientException  Unhandled client error condition has occured.
      * @exception NeoServerException  If there was an internal NeoCore XMS error.
	  *
	  * @see NeoResultProfile
	  * @see NeoCursor
	  * @see NeoResultChunk
	  * @see NeoResultChunk#NeoResultChunkState
	  *
	  * @return The current cursor position after the movement takes place.
	  */
    const __int64 moveNeoCursor(const __int64 movement = 1)
	throw ( NeoInvalidArgumentException,
			NeoInvalidSessionException,
			NeoManualLoadRequiredException,
			NeoDestroyedResultChunkException,
			NeoCursorOutOfRangeException,
			NeoCommunicationException,
			NeoClientException,
			NeoServerException);



	/** 
	  * @memo <DL><DD>Repositions the NeoCursor within this NeoQueryResult.</DD></DL>
      * @doc <br/>Repositions the NeoCursor within this NeoQueryResult. Additionally, if the
	  * NeoResultProfile resultChunkDisacardMode was set to Auto (i.e. 1) and the NeoCursor leaves a
	  * NeoResultChunk that NeoResultChunk will be automatically destroyed. Also, if the
	  * NeoResultProfile resultChunkTransferMode was set to Auto (i.e. 1) and the NeoCursor enters 
	  * into a NeoResultChunk that is in the state of NEVER_LOADED this method will automatically
	  * load the NeoResultChunk. 
	  *
	  * @param absoluteCursorLocation  The absolute location where you want to place the NeoCursor in this NeoQueryResult, the lower
	  * boundary of a NeoQueryResult is always zero.
	  *
	  * @exception NeoInvalidArgument - If movement is equal to zero.
      * @exception NeoInvalidSessionException - If the session that was used for the initial query that constructed 
	  * this NeoQueryResult is no longer valid.
      * @exception NeoManualLoadRequiredException - If the NeoCursor is positioned in a NeoResultChunk that needs loading 
	  * and you are operating in manual-mode. You can obtain the NeoResultChunk's ID that needs to be loaded from the exception. 
      * @exception NeoDestroyedResultChunkException - If the NeoCursor is positioned in a NeoResultChunk that has been destroyed.
      * @exception NeoCursorOutOfRangeException - If an attempt is made to reposition to a cursor position that does not
      * exist within this NeoQueryResult. You can obtain the boundary NeoCursor position from the exception.
      * @exception NeoCommunicationException - If there was a problem communicating with the NeoCore XMS Server to 
	  * load (i.e. auto-mode enabled) a NeoResultChunk.
      * @exception NeoClientException - Unhandled client error condition has occured.
      * @exception NeoServerException - If there was an internal NeoCore XMS error.
	  *
	  * @see NeoResultProfile
	  * @see NeoCursor
	  * @see NeoResultChunk
	  * @see NeoResultChunk#NeoResultChunkState
	  *
	  * @return The current cursor position after the reposition takes place.
	  */
    const __int64 repositionNeoCursor(const __int64 absoluteCursorLocation)
	throw ( NeoInvalidArgumentException,
			NeoInvalidSessionException,
			NeoManualLoadRequiredException,
			NeoDestroyedResultChunkException,
			NeoCursorOutOfRangeException,
			NeoCommunicationException,
			NeoClientException,
			NeoServerException);



    /**
	  * @memo <DL><DD>This should be called to free each result.</DD></DL>
      * @doc <br/>Since the XML returned from methods on the NeoQueryResult are allocated from the heap it 
	  * should be freed by the client when no longer needed, releaseXML is provided to the client 
	  * to accomplish this task	.
	  *
	  * @param xmlBuffer the buffer that you wish to release, this parameter is assigned NULL
	  * after it is released. 
	  *
	  * @return void
	  */
	void releaseXML(char* XMLBuffer);

	

  private:

	  friend class NeoConnection;

	/**
	  * @memo <DL><DD>Constructs a NeoQueryResult and its aggregate object NeoResultChunk(s), and NeoCursor 
	  * using the queryResultXML provided.</DD></DL>
	  * @doc <br/>Constructs a NeoQueryResult and its aggregate object NeoResultChunk(s), and NeoCursor 
	  * using the queryResultXML provided. Attaches a copy of the NeoResultProfile object to this 
	  * NeoQueryResult. Also populates the first NeoResultChunk with XML AxisElements obtained 
	  * from the ChunkXML tag. As we move the NeoCursor or request XML from this object 
	  * communication takes place with the NeoCore XMS Server to load or destroy 
	  * NeoResultChunks as needed.  NeoQueryResults are associated with the session that they 
	  * were created under when the initial queryXML was executed. This object will have a 
	  * different lifetime than the NeoConnection or SessionManaged NeoConnection that created it. 
	  * 
      * 
	  * @param urlConnection An established connection to the NeoCore XMS Server.
	  * @param sid The Session ID that was used when the initial query was executed.
	  * @param queryResultXML The XML returned from the NeoCore XMS Server when the queryXML was
	  * executed.
	  *
      *   NeoQueryResult format: <br/><br/>
      * 
      * 	&lt;Query-Results&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
      * 		&lt;ID&gt;numeric&lt;/ID&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
      * 		&lt;Truncated&gt; 0 | 1 &lt;/Truncated&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
      * 		&lt;SessionID&gt;numeric&lt;/SessionID&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
      * 		&lt;NeoResultChunks&gt;2&lt;/NeoResultChunks&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
      * 		&lt;NeoResultChunk&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
      * 			&lt;ID&gt;0&lt;/ID&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
      * 			&lt;Offset&gt;numeric&lt;/Offset&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
      * 			&lt;AxisElements&gt; numeric  &lt;/AxisElements&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
      *             &lt;NeoCursorLoc&gt;numeric&lt;/NeoCursorLoc&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
      *         &lt;/NeoResultChunk&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
      *         &lt;NeoResultChunk&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
      * 			&lt;ID&gt;1&lt;/ID&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
      * 			&lt;Offset&gt;numeric&lt;/Offset&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
      * 			&lt;AxisElements&gt;numeric&lt;/AxisElements&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
      * 		&lt;/NeoResultChunk&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
      * 		&lt;ChunkXML&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
      * 			&lt;ID&gt;0&lt;/ID&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
      * 			&lt;AE&gt;axisElementData&lt;/AE&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
      * 			&lt;AE&gt;axisElementData&lt;/AE&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
      * 			&lt;AE&gt;axisElementData&lt;/AE&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
      * 			...<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
      * 		&lt;/ChunkXML&gt;<br/>
      * 	&lt;/Query-Results&gt;      
	  * @param resultProfile The result profile that was used when the queryXML was executed
	  *
	  * @see SessionManagedNeoConnection releaseNeoQueryResult
	  * @see NeoConnection releaseNeoQueryResult
	  *
	  * @exception NeoInvalidArgumentException - If any of the parameters are invalid.
      * @exception NeoClientException - Unhandled client-side error condition has occured. 
	  * @exception NeoServerException - Server did not provide correct queryResultXML
	  */
	NeoQueryResult(UrlConnection* urlConnection, const char* const sid,  const char* const queryResultXML, 
				   const NeoResultProfile* const resultProfile) 						
	throw (NeoInvalidArgumentException,
		   NeoClientException,
		   NeoServerException);

    /**
	  * @memo <DL><DD>Takes the initial XML (Metadata and first chunkXML) and creates the objects identified by NeoQueryResult.</DD></DL>
      * @doc <br/>Takes the initial XML (Metadata and first chunkXML) and creates the objects identified by NeoQueryResult.
	  * This is used during construction of a NeoQueryResult.Assumes well-formed XML.
      *
      * @param queryResultXML as follows
      * NeoQueryResult format: 
      *
      *	&lt;/Query-Results&gt;
      *		&lt;ID&gt;numeric&lt;/ID&gt;
      *		&lt;Truncated&gt; 0 | 1 &lt;/Truncated&gt;
      *		&lt;SessionID&gt;numeric&lt;/SessionID&gt;
      *		&lt;NeoResultChunks&gt;2&lt;/NeoResultChunks&gt;
      *		&lt;NeoResultChunk&gt;
      *			&lt;ID&gt;0&lt;/ID&gt;
      *			&lt;Offset&gt;numeric&lt;/Offset&gt;
      *			&lt;AxisElements&gt; numeric  &lt;/AxisElements &gt;
      *         &lt;CursorLoc&gt;numeric&lt;/CursorLoc&gt;
      *      &lt;/NeoResultChunk&gt;
      *      &lt;NeoResultChunk&gt;
      *			&lt;ID&gt;1&lt;/ID&gt;
      *			&lt;Offset&gt;numeric&lt;/Offset&gt;
      *			&lt;AxisElements&gt;numeric&lt;/AxisElements&gt;
      *		&lt;/NeoResultChunk&gt;
      *		&lt;ChunkXML&gt;
      *			&lt;ID&gt;0&lt;/ID&gt;
      *			&lt;AE&gt;axisElementData&lt;/AE&gt;
      *			&lt;AE&gt;axisElementData&lt;/AE&gt;
      *			&lt;AE&gt;axisElementData&lt;/AE&gt;
      *			...
      *		&lt;/ChunkXML&gt;
      *	&lt;/Query-Results>
	  *
	  * @exception NeoInvalidArgumentException - If queryResultXML is NULL or empty.
      * @exception NeoClientException - Unhandled client error condition has occured.
	  * @excepiton NeoServerException - Server did not provide correct queryResultXML.
	  *
	  * @return void
	  *
	  */
	void disassembleNeoQueryResultXML(const char* const queryResultXML)
    throw (NeoInvalidArgumentException,
		   NeoClientException,
		   NeoServerException);




    /** 
	  * @memo <DL><DD>Takes the NeoResultChunkXML and creates the NeoResultChunk objects that are contained by 
	  * this NeoQueryResult.</DD></DL>
      * @doc <br/>Takes the NeoResultChunkXML and creates the NeoResultChunk objects that are contained by 
	  * this NeoQueryResult. Assumes well-formed XML.
	  *
      * @param resultChunks A vector of resultChunks in XML, each position of the vector contains 
	  * the following. the <CursorLoc> tag is only present in the NeoResultChunk that has
	  * the NeoCursor. 
      * 
	  *		&lt;NeoResultChunk&gt;
	  *			&lt;ID&gt;0&lt;/ID&gt;
	  *			&lt;Offset&gt;numeric&lt;/Offset&gt;
	  *			&lt;AxisElements&gt; numeric  &lt;/AxisElements &gt;
	  *         &lt;CursorLoc&gt;numeric&lt;/CursorLoc&gt;
	  *     &lt;/NeoResultChunk&gt;
      * 
	  * @exception NeoInvalidArgumentException - If queryResultXML is NULL or empty
      * @exception NeoClientException - Unhandled client error condition has occured.
	  * @excepiton NeoServerException - Server did not provide correct XML
	  *
	  * @return void
	  */
	void disassembleNeoResultChunkXML(const vector<char*>* resultChunks)
    throw (NeoInvalidArgumentException,
		   NeoClientException,
		   NeoServerException);


    /**
	  * @memo <DL><DD>Destroys all the ResultChunks for this NeoQueryResult, The entire
      * NeoResultChunk object is destroyed (AxisElements and Metadata).</DD></DL>
      * @doc <br/>Destroys all the ResultChunks for this NeoQueryResult, The entire
      * NeoResultChunk object is destroyed (AxisElements and Metadata). May have to
	  * communicate with the Server for destruction if the NeoResultChunks have not 
	  * been transferred to the client. 
	  *
	  * @return void
	  */
    void destroyNeoResultChunks();



    // This ID is the same ID for this NeoQueryResult that exists on the NeoCore XMS Server. 
    int m_ID;


    // The number of AxisElements that make up this NeoQueryResult. Does not distinquish between 
	// loaded and not-loaded. 
    __int64 m_totalAxisElements;

    // The number of NeoResultChunks that make up this NeoQueryResult. Does not distinquish between 
	// loaded and not-loaded. 
    int m_totalNeoResultChunks;


    // This identifies to the client whether or not the result was truncated
    // (i.e. not a full result).  Results will be truncated if the NeoResultProfile defines 
	// exceededResultsMode = truncate result and the result is larger than maxAxisElements.
	//
	// @see NeoResultProfile
    bool m_truncated;


    // The NeoResultChunks that make up this NeoQueryResult
    vector <NeoResultChunk*>* m_theResultChunks;


    //The NeoCursor for this NeoQueryResult
    NeoCursor* m_theCursor;



    // A copy of the original NeoResultProfile that was used to create this NeoQueryResult
    // 
	// @see NeoResultProfile
    NeoResultProfile* m_theResultProfile;



	// The connection to the NeoCore XMS Server, this object is reference counted. We can call
	// delete ont this object even though the NeoConnection or SessionManagedNeoConnection may have
	// already been destroyed.

	UrlConnection* m_urlConnection; 


    // The Session ID, this is used when whe have to communicate with the NeoCore XMS Server to 
	// load or destroy NeoResultChunks.
	char* m_sid; 


	// Since this object contains a considerable amount of state and using it
	// simultaneously from two different threads really doesn't make since we
	// need to serialize this object 
	API_NeoCritical* m_lock;


};


#endif //NEO_QUERYRESULT_H

