//  Copyright 2001-2002, NeoCore Inc.
//  All Rights Reserved

#ifndef SESSION_MANAGED_NEO_CONNECTION_H
#define SESSION_MANAGED_NEO_CONNECTION_H
#ifdef WIN32
#pragma warning( disable : 4290 ) 
#endif

#include "API_LEM.h"
#include "NeoConnection.h"
#include "NeoConnectionPool.h"

#include "NeoResultProfile.h"
#include "NeoQueryResult.h"


/** 
 * @memo <DL><DD>SessionManagedNeoConnection is built on the NeoConnection object.</DD></DL>
 * @doc <br/>SessionManagedNeoConnection is built on the NeoConnection object.
 * SessionManagedNeoConnection will save a users Session Id(SID).
 * It abstracts the connection process, and allows the user
 * to send XPATH type queries and commands to the XMS Server.
 * Developers should use the NeoConnection object,
 * or the SessionManagedNeoConnection object.
 * The SessionManagedNeoConnection class replaces the SessionManagedProxy object, with new signatures.
 * The SessionManagedNeoConnection class throws more Exceptions. <br/><br/>
 *
 * NOTE: If the sid is NULL and AllowAnonymousUser is set to true in the config file,
 * "anonymous user" with "guest" Access Control privileges is assumed. Only query type 
 * methods will return a response in this case, all other methods are unavailable. 
 */
class HTTPCLIENT4_API SessionManagedNeoConnection
{
	friend class NeoACAdmin;
	friend class NeoTransform;

private:
	char* sid;
	NeoConnection* connect;
	NeoConnectionPool* pool;
	/**
	 * @memo <DL><DD>Grabs a connection, either from the pool or from the memeber variable connect.</DD></DL>
     * @doc <br/>Grabs a connection, either from the pool or from the memeber variable connect.
	 */
	NeoConnection* getConnection()
		throw (NeoConnectionPoolException, NeoPoolClosedException);

	/**
	 * @memo <DL><DD>Releases a connection, either to the pool or back to connect.</DD></DL>
     * @doc <br/>Releases a connection, either to the pool or back to connect.
	 */
	void releaseConnection(NeoConnection* c);

	API_LEM* lem;

public:
    /** 
	 * @memo <DL><DD>This constructor attempts to connect to the specified host and port
     * to determine if they are valid.</DD></DL>
     * @doc <br/>This constructor attempts to connect to the specified host and port
     * to determine if they are valid.  If this constructor is used, all messages returned to the
     * client will be in english unless calls to setRootDir and setLanguage are made.
	 * 
     * @param host The host to connect to.
     * @param port The port to connect on.
	 * @exception NeoMalformedURLException - If the host URL is not well-formed.
     * @exception NeoCommunicationException - If NeoConnection could not be made.
     * @exception NeoInvalidArgumentException - If the host is empty/null or the port is invalid.
     */
	SessionManagedNeoConnection (const char* host, const int port) 
	throw (NeoMalformedURLException, 
		   NeoCommunicationException,
		   NeoInvalidArgumentException);

   /**
     * @memo <DL><DD>This constructor will pull connections from a pool whenever they are needed.</DD></DL>
     * @doc <br/>This constructor will pull connections from a pool whenever they are needed.
	 *
     * @param pool The pool that this object will pull connections from.
     * @exception NeoInvalidArgumentException If the pool is null.
	 *
     */
	SessionManagedNeoConnection (NeoConnectionPool* thePool) 
	throw ( NeoInvalidArgumentException);


	/**
	 * @memo <DL><DD>This constructor attempts to connect to the specified host and port
	 * to determine if they are valid. &nbsp;&nbsp;</DD></DL>
	 * @doc <br/>This constructor attempts to connect to the specified host and port
	 * to determine if they are valid. &nbsp;&nbsp;  Use this constructor to specify
	 * a language other than English.  If rootdir is not specified, English will be used
	 * until a call is made to setRootDir.
	 *
	 * @param host The host to connect to.
	* @param port The port to connect on.
	* @param language The desired language expected from the server. (en, ja, etc)
	* @param charset The desired character charset expected from the server. (UTF-8, Shift_JIS, etc)
	* @param rootdir The base directory of the application - used to locate locale properties files
	* @exception NeoMalformedURLException - If the host URL is not well-formed.
	* @exception NeoCommunicationException - If NeoConnection could not be made.
	* @exception NeoInvalidArgumentException - If the host is empty/null.
	*/
	SessionManagedNeoConnection (const char* host, const int port, const char *language, 
			const char *charset, const char *rootdir = NULL) 
	throw (NeoMalformedURLException,
		   NeoCommunicationException, 
		   NeoInvalidArgumentException);

     /**
	 * @memo <DL><DD>Set the preferred language that is expected in server responses.</DD></DL>
	 * @doc <br/>Set the preferred language that is expected in server responses.  .
     *
     * @param language The desired language expected from the server.
     * @exception NeoInvalidArgumentException - If the language pointer is empty/null.
     * @return void
     */
	void setLanguage(const char* language) 
    throw  (NeoInvalidArgumentException);

     /**
	 * @memo <DL><DD>Set the root directory of the client application.</DD></DL>
	 * @doc <br/>Sets the root directory of the client application.  The rootDir is used
	 * to locate the appropriate languages properties file.  Language properties files must
	 * be located in <rootdir>/resource/locales in order to be used.
     *
     * @param language The root directory of the client appliation.
     * @exception NeoInvalidArgumentException - If the rootDir pointer is empty/null.
     * @return void
     */
	void setRootDir(const char* rootDir) 
    throw  (NeoInvalidArgumentException);



     /**
	 * @memo <DL><DD>Set the preferred charset that is expected in server responses.</DD></DL>
	 * @doc <br/>Set the charset language that is expected in server responses.  .
     *
     * @param charset The desired charset expected from the server.
     * @exception NeoInvalidArgumentException - If the charset pointer is empty/null.
     * @return void
     */
	void setCharset(const char* charset) 
    throw  (NeoInvalidArgumentException);

/**
	 * @memo <DL><DD>Get the preferred language that is expected in server responses.</DD></DL>
	 * @doc <br/>Get the preferred language that is expected in server responses.
     *
     * @return Language, the preferred language that is expected in server responses.
     */
	const char* getLanguage();

     /**
	 * @memo <DL><DD>Get the preferred charset that is expected in server responses.</DD></DL>
	 * @doc <br/>Get the charset language that is expected in server responses.
     *
     * @return Charset, the charset language that is expected in server responses.
     */
	const char* getCharset();

	/**
	 * @memo <DL><DD>Cleans up memory and resources allocated by this object, and releases the socket connection.</DD></DL>
     * @doc <br/>Cleans up memory and resources allocated by this object, and releases the socket connection. <br/><br/>
	 * 
     * NOTE ---> This method does NOT call logout, if you have logged in please call logout() before
	 * destructing this object otherwise you will leave an orphan session on the NeoCore
	 * XMS Server. 	 
	 */
	virtual ~SessionManagedNeoConnection (void) ;

   /**
     * @memo <DL><DD>Activates Access Control on the NeoCore XMS.</DD></DL>
     * @doc <br/>Activates Access Control on the NeoCore XMS.<br/><br/>
     *
     * NOTE ---> Before the XMS server can apply Access Control, there must be
     * Access Control documents stored in the server.  The defualt access control
     * document is in the NeoCore/neoxml/config directory. Before calling this method
     * store the DefaultAC.xml document in the config directory.
	 *
     * @exception NeoServerException - If there was an internal NeoCore XMS error.
     * @exception NeoCommunicationException - If there was an error communicating with the host.
     * @exception NeoTransactionException - If any transactional errors occurred.
     * @exception NeoInvalidSessionException - If this objects session ID is not valid.
     * @exception NeoAuthorizationFailedException - If you do not have authorization to make this request.
     * @exception NeoPoolClosedException - If this object is using a pool, and that pool was closed.
     * @exception NeoConnectionPoolException - If this object is using a pool, and no connection were available from that pool.
     * 
     * @return XML, NeoCore XMS's response.
	 *
     */
	char* activateAccessControl() 
    throw  (NeoServerException, 
            NeoCommunicationException,
            NeoTransactionException, 
            NeoInvalidSessionException,
            NeoAuthorizationFailedException,
            NeoPoolClosedException,
            NeoConnectionPoolException);


    /**
	 * @memo <DL><DD>Sends the commit transaction command to the NeoCore XMS.</DD></DL>
     * @doc <br/>Sends the commit transaction command to the NeoCore XMS.
     *
     * @exception NeoServerException - If there was an internal NeoCore XMS error.
     * @exception NeoCommunicationException - If there was an error communicating with the host.
     * @exception NeoTransactionException - If any transactional errors occurred.
     * @exception NeoInvalidSessionException - If this objects session ID is not valid.
     * @exception NeoAuthorizationFailedException - If you do not have authorization to make this request.
     * @exception NeoPoolClosedException - If this object is using a pool, and that pool was closed.
     * @exception NeoConnectionPoolException - If this object is using a pool, and no connection were available from that pool.
	 *
     * @return XML, NeoCore XMS's response.
     */
	char* commitTransaction () 
    throw  (NeoServerException,
            NeoCommunicationException,
            NeoTransactionException,
            NeoInvalidSessionException,
			NeoAuthorizationFailedException, 
            NeoPoolClosedException,
            NeoConnectionPoolException);
 


    /**
	 * @memo <DL><DD>Locks the entire NeoCore XMS database.</DD></DL>
     * @doc <br/>Locks the entire NeoCore XMS database.
     * If you will likely lock most of the database.  It is advisable to lock it ahead of time.
     *
     * @param lockType Is the type of lock desired.  It must be either, SHARED, UPDATE or EXCLUSIVE
     * @exception NeoServerException - If there was an internal NeoCore XMS error.
     * @exception NeoCommunicationException - If there was an error communicating with the host.
     * @exception NeoTransactionException - If any transactional errors occurred.
     * @exception NeoInvalidArgumentException - If the session id is blank or null, or if the lockType 
     * is not one of the three acceptable types.
     * @exception NeoInvalidSessionException - If this objects session ID is not valid.
     * @exception NeoAuthorizationFailedException - If you do not have authorization to make this request.
     * @exception NeoPoolClosedException - If this object is using a pool, and that pool was closed.
     * @exception NeoConnectionPoolException - If this object is using a pool, and no connection were available from that pool.
     * @return XML, NeoCore XMS's response
     */
    char* globalLock(const char* lockType) 
    throw  (NeoServerException, 
            NeoCommunicationException,
            NeoTransactionException,
            NeoInvalidArgumentException,
            NeoInvalidSessionException,           
            NeoAuthorizationFailedException,
            NeoPoolClosedException,
            NeoConnectionPoolException);

	
	// Administrative commands
    /**
	 * @memo <DL><DD>Log onto the NeoCore XMS database instance.</DD></DL>
     * @doc <br/>Log onto the NeoCore XMS database instance.
     *
     * @param user The user's username.
     * @param password The user's password.
     * @exception NeoServerException - If there was an internal NeoCore XMS error.
     * @exception NeoCommunicationException - If there was an error communicating with the host.
     * @exception NeoTransactionException - If any transactional errors occurred.
     * @exception NeoInvalidArgumentException - If either the user or password is invalid.
     * @exception NeoAuthorizationFailedException - If you do not have authorization to make this request or your
     * request is denied access.
     * @exception NeoPoolClosedException - If this object is using a pool, and that pool was closed.
     * @exception NeoConnectionPoolException - If this object is using a pool, and no connection were available from that pool.
     * @return the SessionManagedNeoConnection SID (Session ID)
     */
	char* login(const char* user, const char* password) 
    throw  (NeoServerException,
            NeoCommunicationException,     
            NeoTransactionException,
            NeoInvalidArgumentException,
            NeoAuthorizationFailedException,
            NeoPoolClosedException,
            NeoConnectionPoolException);


																		 
   /**
     * @memo <DL><DD>Logout of the NeoCore XMS database instance.</DD></DL>
     * @doc <br/>Logout of the NeoCore XMS database instance.
     *
     * @exception NeoServerException - If there was an internal NeoCore XMS error.
     * @exception NeoCommunicationException - If there was an error communicating with the host.
     * @exception NeoTransactionException - If any transactional errors occurred.
     * @exception NeoInvalidSessionException - If this objects session ID is not valid.
     * @exception NeoAuthorizationFailedException - If you do not have authorization to make this request.
     * @exception NeoPoolClosedException - If this object is using a pool, and that pool was closed.
     * @exception NeoConnectionPoolException - If this object is using a pool, and no connection were available from that pool.
     * @return XML, NeoCore XMS's response.
     */
	char* logout()
    throw  (NeoServerException, 
            NeoCommunicationException,    
            NeoTransactionException,    
            NeoInvalidSessionException,
            NeoAuthorizationFailedException,
            NeoPoolClosedException,
            NeoConnectionPoolException);


    /**
 	 * @memo <DL><DD>Determines if this object has a Session ID and whether or not it is logged in, (i.e. a valid SID).</DD></DL>
     * @doc <br/>Determines if this object has a Session ID and whether or not it is logged in, (i.e. a valid SID).
     *
     * @exception NeoServerException - If there was an internal server error.
     * @exception NeoCommunicationException - If there was an error communicating with the host.
     * @exception NeoTransactionException - If any transactional errors occurred.
     * @exception NeoInvalidSessionException - If this objects session ID is not valid.
     * @exception NeoAuthorizationFailedException - If you do not have authorization to make this request 
     * @exception NeoPoolClosedException - If this object is using a pool, and that pool was closed.
     * @exception NeoConnectionPoolException - If this object is using a pool, and no connection were available from that pool.
	 *
     * @return true if this object has a session ID and that session is valid (i.e. active),
     * false otherwise.
     */
	bool isLoggedIn() 
    throw  (NeoServerException, 
            NeoCommunicationException,
            NeoTransactionException, 
            NeoInvalidSessionException,
            NeoAuthorizationFailedException,
            NeoPoolClosedException,
            NeoConnectionPoolException);



    /**
	 * @memo <DL><DD>Set password on a user.</DD></DL>
	 * @doc <br/>Set password on a user.  Requires that the logged in user has administrative rights.
     *
     * @param user The user's username.
     * @param password The user's password.
     * @exception NeoServerException - If there was an internal NeoCore XMS error.
     * @exception NeoCommunicationException - If there was an error communicating with the host.
     * @exception NeoTransactionException - If any transactional errors occurred.
     * @exception NeoInvalidArgumentException - If either the user or password is invalid.
     * @exception NeoInvalidSessionException - If this objects session ID is not valid.
     * @exception NeoAuthorizationFailedException - If you do not have authorization to make this request.
     * @exception NeoPoolClosedException - If this object is using a pool, and that pool was closed.
     * @exception NeoConnectionPoolException - If this object is using a pool, and no connection were available from that pool.
     * @return XML, NeoCore XMS's response.     
     */
	char* setPassword(const char* user, const char* password) 
    throw  (NeoServerException,
            NeoCommunicationException,
            NeoTransactionException,
            NeoInvalidArgumentException,
            NeoInvalidSessionException,
            NeoAuthorizationFailedException,
            NeoPoolClosedException,
            NeoConnectionPoolException);


    /** 
	 * @memo <DL><DD>Sets the trace levels for the Error Log System. </DD></DL>
     * @doc <br/>Sets the trace levels for the Error Log System. This method will
     * replace the current trace level setting. Previous settings are not preseved.
     * The Error Log System will maintain the values FATAL:LOG_all;ERR:LOG_all;WARNING:LOG_all
     * if they are not specified.
     *
     * @param tracelvl  The tracelevel to use. <br/>
     * This should be in the form: Severity:level[,level];Severity:level[,level]
     * INFO:LOG_Performance,LOG_Query is equivalent to INFO:LOG_Performance;INFO:LOG_Query
     * Set tracelvl to the empty string to revert the system trace level to
     * FATAL:LOG_all;ERR:LOG_all;WARNING:LOG_all.
	 *
     * @exception NeoServerException - If there was an internal NeoCore XMS error.
     * @exception NeoCommunicationException - If there was an error communicating with the host.
     * @exception NeoTransactionException - If any transactional errors occurred.
     * @exception NeoInvalidArgumentException - If the tracelvl is invalid.
     * @exception NeoInvalidSessionException - If this objects session ID is not valid.
     * @exception NeoAuthorizationFailedException - If you do not have authorization to make this request.
     * @exception NeoPoolClosedException - If this object is using a pool, and that pool was closed.
     * @exception NeoConnectionPoolException - If this object is using a pool, and no connection were available from that pool.
	 * 
     * @return The NeoCore XMS's response.
	 *
     */
	char* setTraceLevels(const char* tracelvl) 
    throw  (NeoServerException, 
            NeoCommunicationException,
            NeoTransactionException, 
            NeoInvalidArgumentException,
            NeoInvalidSessionException,             
            NeoAuthorizationFailedException,
            NeoPoolClosedException,
            NeoConnectionPoolException); 


    /** 
	 * @memo <DL><DD>Returns the current tracelevel string being used by the Error Log.</DD></DL>
     * @doc <br/>Returns the current tracelevel string being used by the Error Log.
     *
     * @exception NeoServerException - If there was an internal NeoCore XMS error.
     * @exception NeoCommunicationException - If there was an error communicating with the host.
     * @exception NeoTransactionException - If any transactional errors occurred.
     * @exception NeoInvalidSessionException - If this objects session ID is not valid..
     * @exception NeoAuthorizationFailedException - If you do not have authorization to make this request.
     * @exception NeoPoolClosedException - If this object is using a pool, and that pool was closed.
     * @exception NeoConnectionPoolException - If this object is using a pool, and no connection were available from that pool.
	 *
     * @return XML containing the current trace levels on the NeoCore XMS.
     */
	char* getTraceLevels() 
    throw  (NeoServerException, 
            NeoCommunicationException,
            NeoTransactionException, 
            NeoInvalidSessionException,            
            NeoAuthorizationFailedException,
            NeoPoolClosedException,
            NeoConnectionPoolException);


    /**
	 * @memo <DL><DD>Stores an XML File.</DD></DL>
     * @doc <br/>Stores an XML File.
     *
     * @param sourceXMLFile (Required) filename of the document containing the XML to be stored
     * @param schemaURI (Optional, NULL if not provided) contains the reference name of the 
	 * schema that is stored.
     * @param prefixXMLFile (optional, NULL if not provided) filename of the document 
	 * containing the Meta-Data of the XML being stored.
	 *
     * @exception NeoServerException - If there was an internal NeoCore XMS error.
     * @exception NeoCommunicationException - If there was an error communicating with the host.
     * @exception NeoTransactionException - If any transactional errors occurred.
     * @exception NeoInvalidArgumentException - If the source file is null, or empty.
     * @exception NeoNoFileFoundException - If any of the files cannot be located.
     * @exception NeoMalformedXMLException - If the provided XML is not valid.
     * @exception NeoInvalidSessionException - If this objects session ID is not valid.
     * @exception NeoAuthorizationFailedException - If you do not have authorization to make this request
     * @exception NeoPoolClosedException - If this object is using a pool, and that pool was closed.
     * @exception NeoConnectionPoolException - If this object is using a pool, and no connection were available from that pool.
     * @see XMLUtils#getStores
     * @see XMLUtils#getStoreLastDocID
	 *
     * @return XML message from the XMS Server with the document id if successful.
     */
	char* storeFileXML(const char *sourceXMLFile,const  char *schemaURI, const char *prefixXMLFile)   
    throw  (NeoServerException,
            NeoCommunicationException,
            NeoTransactionException,
            NeoInvalidArgumentException,
            NeoNoFileFoundException,
            NeoMalformedXMLException,
            NeoInvalidSessionException,
            NeoAuthorizationFailedException,
            NeoPoolClosedException,
            NeoConnectionPoolException);




    /**
     * @memo <DL><DD>This method will store the XML in the argument strings to the NeoCore XMS.</DD></DL>
     * @doc <br/>This method will send the XML in the argument strings to the NeoCore XMS.
	 *
     * @param sourceXML XMLData containing the XML to be stored.
     * @param schemaURI contains the reference name of the schema that is stored.
     * @param prefixXML XMLData containing the Meta-Data for the XML to be stored.
     * @exception NeoServerException - If there was an internal NeoCore XMS error.
     * @exception NeoCommunicationException If there was an error communicating with the host.
     * @exception NeoTransactionException - If any transactional errors occurred.
     * @exception NeoInvalidArgumentException - If the source file is null, or empty.
     * @exception NeoMalformedXMLException - If the provided XML is not valid.
     * @exception NeoInvalidSessionException - If this objects session ID is not valid.
     * @exception NeoAuthorizationFailedException - If you do not have authorization to make this request.
     * @exception NeoPoolClosedException - If this object is using a pool, and that pool was closed.
     * @exception NeoConnectionPoolException - If this object is using a pool, and no connection were available from that pool.
     * @see XMLUtils#getStores
     * @see XMLUtils#getStoreLastDocID
     * @return XML success message with the document id
     */
	char* storeXML(const char *sourceXML, const char *schemaURI, const char *prefixXML)
    throw  (NeoServerException, 
            NeoCommunicationException,
            NeoTransactionException,
            NeoInvalidArgumentException,
            NeoMalformedXMLException,
            NeoInvalidSessionException,
            NeoAuthorizationFailedException,
            NeoPoolClosedException,
            NeoConnectionPoolException);


    /**
	 * @memo <DL><DD>Copy XML duplicates nodes matching a query.</DD></DL>
     * @doc <br/>Copy XML duplicates nodes matching a query.
     *
     * @param query XPath expression that uniquely identifies the document nodes
     * to be copied.
     * @exception NeoServerException - If there was an internal server error.
     * @exception NeoCommunicationException - If there was an error communicating with the host.
     * @exception NeoTransactionException - If any transactional errors occurred.
     * @exception NeoMalformedXPathException - If the provided XPath is not valid.
     * @exception NeoInvalidArgumentException - If the query is null, or empty.
     * @exception NeoInvalidSessionException - If this objects session ID is not valid.
     * @exception NeoAuthorizationFailedException - If you do not have authorization to make this request.
     * @exception NeoPoolClosedException - If this object is using a pool, and that pool was closed.
     * @exception NeoConnectionPoolException - If this object is using a pool, and no connection were available from that pool.
     * @see XMLUtils#getCopyNumber
     * @return XML containing the copy number of the document copied.    
     */
	char* copyXML(const char *query) 
    throw  (NeoServerException,
            NeoCommunicationException,
            NeoTransactionException,            
            NeoMalformedXPathException,
            NeoInvalidArgumentException,
            NeoInvalidSessionException,
            NeoAuthorizationFailedException,
            NeoPoolClosedException,
            NeoConnectionPoolException); 


   /**
     * @memo <DL><DD>Deletes XML on the NeoCore XMS.</DD></DL>
     * @doc <br/>Deletes XML on the NeoCore XMS.  The NeoCore XMS will delete 
	 * all XML fragments or documents that match the given XPath.
     *
     * @param query An XPath expression identifying the node(s) to be deleted.
     * @exception NeoServerException - If there was an internal NeoCore XMS error.
     * @exception NeoCommunicationException - If there was an error communicating with the host.
     * @exception NeoTransactionException - If any transactional errors occurred.
     * @exception NeoInvalidArgumentException - If the query is null, or empty.
     * @exception NeoMalformedXPathException - If the provided XPath is not valid.
     * @exception NeoInvalidSessionException - If this objects session ID is not valid.
     * @exception NeoAuthorizationFailedException - If you do not have authorization to make this request.
     * @exception NeoPoolClosedException - If this object is using a pool, and that pool was closed.
     * @exception NeoConnectionPoolException - If this object is using a pool, and no connection were available from that pool.
     * @see XMLUtils#getDeletions
     * @return XML containing the total number of nodes deleted.
     */
	char* deleteXML(const char *query) 
    throw  (NeoServerException,
            NeoCommunicationException,
            NeoTransactionException,  
            NeoInvalidArgumentException,
            NeoMalformedXPathException,
            NeoInvalidSessionException,
            NeoAuthorizationFailedException,
            NeoPoolClosedException,
            NeoConnectionPoolException);



   /**
     * @memo <DL><DD>Calls the NeoCore XMS with the Query command.</DD></DL>
     * @doc <br/>Calls the NeoCore XMS with the Query command.
     *
     * @param query The XPath or Xquery statement.
     * @exception NeoServerException - If there was an internal server error.
     * @exception NeoCommunicationException - If there was an error communicating with the host.
     * @exception NeoTransactionException - If any transactional errors occurred.
     * @exception NeoInvalidArgumentException - If the query is blank or null.
     * @exception NeoMalformedXPathException - If the supplied XPath query is not valid.
     * @exception NeoInvalidSessionException - If this objects session ID is not valid.
     * @exception NeoPoolClosedException - If this object is using a pool, and that pool was closed.
     * @exception NeoConnectionPoolException - If this object is using a pool, and no connection were available from that pool.
	 *
     * @return XML containing the query results
     */
	char* queryXML(const char *query) 
    throw  (NeoServerException,
            NeoCommunicationException,
            NeoTransactionException,            
            NeoMalformedXPathException,
            NeoInvalidArgumentException,
            NeoInvalidSessionException,
            NeoPoolClosedException,
            NeoConnectionPoolException);
	
   /**
     * @memo <DL><DD>Calls the NeoCore XMS with the Query command with Update Intent</DD></DL>
     * @doc <br/>Calls the NeoCore XMS with the Query command with Update Intent meaning that the XML identified by the
     * query is locked during the query (Implicit Transaction) or until a Commit or Rollback is executed
     * (Explicit Transactions).
     *
     * @param query The XPath or XQuery statement.
     * @exception NeoServerException - If there was an internal NeoCore XMS error.
     * @exception NeoCommunicationException - If there was an error communicating with the host.
     * @exception NeoTransactionException - If any transactional errors occurred.
     * @exception NeoInvalidArgumentException - If the query is blank or null.
     * @exception NeoMalformedXPathException - If the supplied XPath query is not valid.
     * @exception NeoInvalidSessionException - If this objects session ID is not valid.
     * @exception NeoAuthorizationFailedException - If you do not have authorization to make this request
     * @exception NeoPoolClosedException - If this object is using a pool, and that pool was closed.
     * @exception NeoConnectionPoolException - If this object is using a pool, and no connection is available from that pool.
	 *
     * @return XML containing the query results.
     */
	char* queryXMLUpdateIntent(const char *query) 
    throw  (NeoServerException,
            NeoCommunicationException,
            NeoTransactionException,            
            NeoMalformedXPathException,
            NeoInvalidArgumentException,
            NeoInvalidSessionException,
            NeoAuthorizationFailedException,
            NeoPoolClosedException,
            NeoConnectionPoolException);

   /**
     * @memo <DL><DD>Queries XMS with cursoring and chunking support.</DD></DL>
     * @doc <br/>Queries XMS with cursoring and chunking support.
     * You must be logged in to use this method.
	 *
     * @param query the XPath query 
     * @param profile determines the profile of your NeoQueryResult
     * @return a NeoQueryResult 
     * @exception NeoServerException - If there was an internal NeoCore XMS error.
     * @exception NeoCommunicationException - If there was an error communicating with the host.
     * @exception NeoTransactionException - If any transactional errors occurred.
     * @exception NeoInvalidArgumentException - If the query is blank or null.
     * @exception NeoMalformedXPathException - If the supplied XPath query is not valid.
     * @exception NeoInvalidSessionException - If this objects session ID is not valid.
     * @exception NeoAuthorizationFailedException - If you do not have authorization to make this request.
     *            Note: anonymous user and guest are not allowed to excecute this command
	 * @exception NeoQueryResultExceededMaxException - If the NeoResultProfile specifies a maxResultAxisElements that
	 *            is volated by this query and they have the NeoResultProfile exceededResultsMode set to REJECT (0).
	 * @exception NeoPoolClosedException - If this object is using a pool and that pool was closed.
	 * @exception NeoConnectionPoolException - If this object is using a pool and there were no available connections.
	 * 
	 * @see NeoResultProfile
     * @see NeoQueryResult
	 * @see releaseNeoQueryResult
	 *
	 * @returns a NeoQueryResult if the result is not empty, otherwise returns NULL signifying an
	 * empty result. 
     */
	NeoQueryResult* queryXML(const char *query, const NeoResultProfile* const profile) 
    throw  (NeoServerException,
            NeoCommunicationException,
            NeoTransactionException,            
            NeoMalformedXPathException,
            NeoInvalidArgumentException,
            NeoInvalidSessionException,
			NeoAuthorizationFailedException,
		    NeoQueryResultExceededMaxException,
            NeoPoolClosedException,
            NeoConnectionPoolException);

	
	/**
	 * @memo <DL><DD>Calls the NeoCore XMS with the Query command with Update Intent.</DD></DL>
     * @doc <br/>Calls the NeoCore XMS with the Query command with Update Intent.
     * Calls NeoCore XMS with the Query command with Update Intent meaning that the XML identified by the 
     * query is locked during the query (Implicit Transaction) or until a Commit or Rollback is executed
     * (Explicit Transactions). You must be logged in to use this method. 
	 *
     * @param query  The XPath or XQuery statement.
     * @param resultProfile  Determines the profile of your NeoQueryResult.
     * @returns  A QueryResult object.
     * @exception NeoServerException - If there was an internal NeoCore XMS error.
     * @exception NeoCommunicationException - If there was an error communicating with the host.
     * @exception NeoTransactionException - If any transactional errors occurred.
     * @exception NeoInvalidArgumentException - If the query is blank or null.
     * @exception NeoMalformedXPathException - If the supplied XPath query is not valid.
     * @exception NeoInvalidSessionException - If this objects session ID is not valid.
     * @exception NeoAuthorizationFailedException - If you do not have authorization to make this request.
     *            Note: anonymous user and guest are not allowed to excecute this command.
	 * @exception NeoQueryResultExceededMaxException - If the NeoResultProfile specifies a maxResultAxisElements that
	 *            is violated by this query and they have the NeoResultProfile exceededResultsMode set to REJECT (0).
	 * @exception NeoPoolClosedException - If this object is using a pool and that pool was closed.
	 * @exception NeoConnectionPoolException - If this object is using a pool and there were no available connections.
	 * 
	 * @see NeoResultProfile
     * @see NeoQueryResult
	 * @see releaseNeoQueryResult
	 *
	 * @returns A NeoQueryResult if the result is not empty, otherwise returns NULL signifying an
	 * empty result. 
	 *
	 */
	NeoQueryResult* queryXMLUpdateIntent(const char *query, const NeoResultProfile* const profile) 
    throw  (NeoServerException,
            NeoCommunicationException,
            NeoTransactionException,            
            NeoMalformedXPathException,
            NeoInvalidArgumentException,
            NeoInvalidSessionException,
			NeoAuthorizationFailedException,
		    NeoQueryResultExceededMaxException,
            NeoPoolClosedException,
            NeoConnectionPoolException);


   /**
	 * @memo <DL><DD>Queries NeoCore XMS and returns the XML result in flattened form.</DD></DL>
     * @doc <br/>Queries NeoCore XMS and returns the XML result in flattened form.  I.E. ND>html>body>p>foobar
     *
     * @param query  The XPath or XQuery statement.
     * @exception NeoServerException - If there was an internal NeoCore XMS error.
     * @exception NeoCommunicationException - If there was an error communicating with the host.
     * @exception NeoTransactionException - If any transactional errors occurred.
     * @exception NeoInvalidArgumentException - If the query is blank or null.
     * @exception NeoMalformedXPathException - If the supplied XPath query is not valid.
     * @exception NeoInvalidSessionException - If this objects session ID is not valid.
     * @exception NeoPoolClosedException - If this object is using a pool, and that pool was closed.
     * @exception NeoConnectionPoolException - If this object is using a pool, and no connection is 
	 * available from that pool.
     * @return XML containing the query results in the flattented form.
     */
	char* queryFlatXML(const char *query) 
    throw  (NeoServerException,           
            NeoCommunicationException,           
            NeoTransactionException, 
            NeoInvalidArgumentException, 
            NeoMalformedXPathException, 
            NeoInvalidSessionException,
            NeoPoolClosedException,
            NeoConnectionPoolException);


   /**
     * @memo <DL><DD>Queries NeoCore XMS and returns the XML result in flattened form with Update Intent.</DD></DL>
     * @doc <br/>Queries NeoCore XMS and returns the XML result in flattened form. Update Intent means 
	 * that the XML identified by the query is locked during the query (Implicit Transaction) or until 
	 * a Commit or Rollback is executed (Explicit Transactions).
     *
     * @param query The XPath or XQuery statement.
     * @exception NeoServerException - If there was an internal NeoCore XMS error.
     * @exception NeoCommunicationException - If there was an error communicating with the host.
     * @exception NeoTransactionException - If any transactional errors occurred.
     * @exception NeoInvalidArgumentException - If the query is blank or null.
     * @exception NeoMalformedXPathException - If the supplied XPath query is not valid.
     * @exception NeoInvalidSessionException - If this objects session ID is not valid.
     * @exception NeoAuthorizationFailedException - If you do not have authorization to make this request
     * @exception NeoPoolClosedException - If this object is using a pool, and that pool was closed.
     * @exception NeoConnectionPoolException - If this object is using a pool, and no connection is 
	 * available from that pool.
     * @return XML containing the query results in the flattented form.
     */
	char* queryFlatXMLUpdateIntent(char *query) 
    throw  (NeoServerException,           
            NeoCommunicationException,           
            NeoTransactionException, 
            NeoInvalidArgumentException, 
            NeoMalformedXPathException, 
            NeoInvalidSessionException,
            NeoAuthorizationFailedException,
            NeoPoolClosedException,
            NeoConnectionPoolException);



   /**
	 * @memo <DL><DD>Counts the number of nodes in the target query.</DD></DL>
     * @doc <br/>Counts the number of nodes in the target query.
     *
     * @param query The XPath or XQuery statement.
     * @param acid  Passing true causes NeoCore XMS to lock the target query during this count transaction, 
     * false indicates that this is a potential dirty-read (not acid, not locked), use false only 
     * when you are certain that this command is excecuted simultaneously with read-only transactions 
     * so that the returned data is correct.
	 * 
     * @exception NeoServerException - If there was an internal NeoCore XMS error.
     * @exception NeoCommunicationException - If there was an error communicating with the host.
     * @exception NeoTransactionException - If any transactional errors occurred.
     * @exception NeoInvalidArgumentException - If the query is blank or null.
     * @exception NeoMalformedXPathException - If the supplied XPath query is not valid.
     * @exception NeoInvalidSessionException - If this objects session ID is not valid.
     * @exception NeoPoolClosedException - If this object is using a pool, and that pool was closed.
     * @exception NeoConnectionPoolException - If this object is using a pool, and no connection is available from that pool.
     * @see XMLUtils#getCount 
     * @return XML containing the count 
     */
	char* queryCountXML(const char *query, const bool acid = true) 
    throw  (NeoServerException,           
            NeoCommunicationException,           
            NeoTransactionException, 
            NeoInvalidArgumentException, 
            NeoMalformedXPathException, 
            NeoInvalidSessionException,
            NeoPoolClosedException,
            NeoConnectionPoolException);

   /**
  	 * @memo <DL><DD>Queries for the names of the nodes one level below the target query.</DD></DL>
     * @doc <br/>Queries for the names of the nodes one level below the target query.
     *
     * @param query The XPath query.
     * @exception NeoServerException - If there was an internal NeoCore XMS error.
     * @exception NeoCommunicationException - If there was an error communicating with the host.
     * @exception NeoTransactionException - If any transactional errors occurred.
     * @exception NeoInvalidArgumentException - If the query is blank or null.
     * @exception NeoMalformedXPathException - If the supplied XPath query is not valid.
     * @exception NeoInvalidSessionException - If this objects session ID is not valid.
     * @exception NeoPoolClosedException - If this object is using a pool, and that pool was closed.
     * @exception NeoConnectionPoolException - If this object is using a pool, and no connection is available from that pool.
     * @return XML containing the names of the nodes one level below the target query.
     * 
     */
	char* queryTreeXML(const char *query) 
    throw  (NeoServerException,           
            NeoCommunicationException,           
            NeoTransactionException, 
            NeoInvalidArgumentException, 
            NeoMalformedXPathException, 
            NeoInvalidSessionException,
            NeoPoolClosedException,
            NeoConnectionPoolException);

   /**
   	 * @memo <DL><DD>Queries data for its context.</DD></DL>
	 * @doc <br/>Queries data for its context. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
     *		Note: Full indexing is required on the database in order to support this command. 
     *
     * @param data  The data to query the context for, must be double quoted (e.g. "DataToLookFor")
     * @exception NeoServerException   If there was an internal NeoCore XMS error.
     * @exception NeoCommunicationException   If there was an error communicating with the host.
     * @exception NeoTransactionException   If any transactional errors occurred.
     * @exception NeoInvalidArgumentException   If the data is blank or null.
     * @exception NeoMalformedXPathException   If the data is not in the correct quoted syntax "" 
     * @exception NeoInvalidSessionException   If this objects session ID is not valid.
     * @exception NeoPoolClosedException   If this object is using a pool, and that pool was closed.
     * @exception NeoConnectionPoolException   If this object is using a pool, and no connection is available from that pool.
     * @return XML containing metadata as a flattened line for each node containing the target query data.
	 *
     */
	char* queryDataContextXML(const char *data) 
    throw  (NeoServerException,           
            NeoCommunicationException,           
            NeoTransactionException, 
            NeoInvalidArgumentException, 
            NeoMalformedXPathException, 
            NeoInvalidSessionException,
            NeoPoolClosedException,
            NeoConnectionPoolException);

    /**
	 * @memo <DL><DD>Gets the current NeoCore XMS Server Statistics.</DD></DL>
     * @doc <br/>Gets the current NeoCore XMS Server Statistics.
     *
     * @param statisticType The type of server statistics that your are requesting.
     * Valid values are:  ALL = 0, ADMIN = 1, STORAGE = 2, ACCESS = 3, BUFFER = 4, TRANSACTION = 5, WINDOW = 6
     * @exception NeoServerException - If there was an internal NeoCore XMS error.
     * @exception NeoCommunicationException - If there was an error communicating with the host.
     * @exception NeoTransactionException - If any transactional errors occurred.
     * @exception NeoInvalidArgumentException - If statisticType is invalid
     * @exception NeoInvalidSessionException - If this objects session ID is not valid.
     * @exception NeoAuthorizationFailedException - If you do not have authorization to make this request.
     * @exception NeoPoolClosedException - If this object is using a pool, and that pool was closed.
     * @exception NeoConnectionPoolException - If this object is using a pool, and no connection is available from that pool.
     * @return XML containing the current NeoCore XMS Satistics
     */    
	char* getServerStatistics(const NeoConnection::SERVER_STATISTIC_TYPES statisticType) 
    throw (NeoServerException,
           NeoCommunicationException, 
           NeoTransactionException, 
           NeoInvalidArgumentException, 
           NeoInvalidSessionException,
           NeoAuthorizationFailedException,
           NeoPoolClosedException,
           NeoConnectionPoolException);

    /**
	 * @memo <DL><DD>Clears the NeoCore XMS Server Statistics and returns the cleared statistics.</DD></DL>
     * @doc <br/>Clears the NeoCore XMS Server Statistics and returns the cleared statistics.
     *
     * @exception NeoServerException - If there was an internal NeoCore XMS error.
     * @exception NeoCommunicationException - If there was an error communicating with the host.
     * @exception NeoTransactionException - If any transactional errors occurred.
     * @exception NeoInvalidSessionException - If this objects session ID is not valid.
     * @exception NeoAuthorizationFailedException - If you do not have authorization to make this request
     * @exception NeoPoolClosedException - If this object is using a pool, and that pool was closed.
     * @exception NeoConnectionPoolException - If this object is using a pool, and no connection is available from that pool.
     * @return XML containing the current NeoCore XMS Satistics after they are cleared
     */
	char* clearServerStatistics() 
    throw  (NeoServerException,
            NeoCommunicationException,
            NeoTransactionException,
            NeoInvalidSessionException,
            NeoAuthorizationFailedException,
            NeoPoolClosedException,
            NeoConnectionPoolException);


    /**
     * @memo <DL><DD>Gets the NeoCore XMS Version.</DD></DL>
     * @doc <br/>Gets the NeoCore XMS Version.
     *
     * @exception NeoServerException - If there was an internal NeoCore XMS error.
     * @exception NeoCommunicationException - If there was an error communicating with the host.
     * @exception NeoTransactionException - If any transactional errors occurred.
     * @exception NeoInvalidSessionException - If this objects session ID is not valid.
     * @exception NeoAuthorizationFailedException - If you do not have authorization to make this request
     * @exception NeoPoolClosedException - If this object is using a pool, and that pool was closed.
     * @exception NeoConnectionPoolException - If this object is using a pool, and no connection is available from that pool.
     * @return XML containing the current NeoCore XMS Version 
     */
	char* getServerVersion()  
    throw  (NeoServerException,
            NeoCommunicationException,
            NeoTransactionException,
            NeoInvalidSessionException,
            NeoAuthorizationFailedException,
            NeoPoolClosedException,
            NeoConnectionPoolException);

    /**
     * @memo <DL><DD>Insert a XML fragment into a specific set of nodes on the NeoCore XMS.</DD></DL>
     * @doc <br/>Insert a XML fragment into a specific set of nodes on the NeoCore XMS.
     *
     * @param query An XPath or XQuery expression identifying the beginning tag(s) after
     * which the insertion will occur.
     * @param insertXML XML fragment to be inserted
     * @exception NeoServerException - If there was an internal NeoCore XMS error.
     * @exception NeoCommunicationException - If there was an error communicating with the host.
     * @exception NeoTransactionException - If any transactional errors occurred.
     * @exception NeoInvalidArgumentException - If either the query or the XML is blank or null.
     * @exception NeoMalformedXPathException - If the supplied XPath query is not valid.
     * @exception NeoMalformedXMLException - If the provided XML is not valid.
     * @exception NeoInvalidSessionException - If this objects session ID is not valid.
     * @exception NeoAuthorizationFailedException - If you do not have authorization to make this request.
     * @exception NeoPoolClosedException - If this object is using a pool, and that pool was closed.
     * @exception NeoConnectionPoolException - If this object is using a pool, and no connection were available from that pool.
     * @see XMLUtils#getInserts
     * @return XML containing the total number of nodes inserted.
	 *
     */
	char* insertXML(const char *query, const char *newXML) 
    throw  (NeoServerException,
            NeoCommunicationException,
            NeoTransactionException,            
            NeoInvalidArgumentException,            
            NeoMalformedXPathException,
            NeoMalformedXMLException,
            NeoInvalidSessionException,
            NeoAuthorizationFailedException,
            NeoPoolClosedException,
            NeoConnectionPoolException);


    /**
     * @memo <DL><DD>Modifies specified node(s).</DD></DL>
     * @doc <br/>Modifies specified node(s).
     *
     *
     * @param query an XPath expression identifying the node(s) to be modified,
     * each node must be an attribute or an element
     * @param modifyXML XML fragment that will replace the text or attribute
     * value associated with the XPath-identified node
     * @exception NeoServerException - If there was an internal server error.
     * @exception NeoCommunicationException - If there was an error communicating with the host.
     * @exception NeoTransactionException - If any transactional errors occurred.
     * @exception NeoInvalidArgumentException - If either the query or XML is null, or empty.
     * @exception NeoMalformedXPathException - If the provided XPath is not valid.
     * @exception NeoMalformedXMLException - If the provided XML is not valid.
     * @exception NeoInvalidSessionException - If this objects session ID is not valid.
     * @exception NeoAuthorizationFailedException - If you do not have authorization to make this request
     * @exception NeoPoolClosedException - If this object is using a pool, and that pool was closed.
     * @exception NeoConnectionPoolException - If this object is using a pool, and no connection were available from that pool.
     * @see XMLUtils#getModifications
     * @return XML containing the total number of nodes modified.    
     */
	char* modifyXML(const char *query, const char *newXML)  
    throw  (NeoServerException,
            NeoCommunicationException,
            NeoTransactionException,
            NeoInvalidArgumentException,
            NeoMalformedXPathException,
            NeoMalformedXMLException,
            NeoInvalidSessionException,
            NeoAuthorizationFailedException,
            NeoPoolClosedException,
            NeoConnectionPoolException);



	// Transaction Management
    /**
     * @memo <DL><DD>Sends the startTransaction command to the NeoCore XMS.</DD></DL>
	 * @doc <br/>Sends the startTransaction command to the NeoCore XMS.
     * This method will use the flush parameter in the config file
     * on the NeoCore XMS.
     *
     * @exception NeoServerException - If there was an internal NeoCore XMS error.
     * @exception NeoCommunicationException - If there was an error communicating with the host.
     * @exception NeoTransactionException - If any transactional errors occurred.
     * @exception NeoInvalidSessionException - If this objects session ID is not valid.
     * @exception NeoAuthorizationFailedException - If you do not have authorization to make this request.
     * @exception NeoPoolClosedException - If this object is using a pool, and that pool was closed.
     * @exception NeoConnectionPoolException - If this object is using a pool, and no connection were available from that pool.
     * @return XML NeoCore XMS's response.
     */
	char* startTransaction () 
    throw  (NeoServerException,
            NeoCommunicationException,
            NeoTransactionException,
            NeoInvalidSessionException,
            NeoAuthorizationFailedException,
            NeoPoolClosedException,
            NeoConnectionPoolException); 


   /**
     * @memo <DL><DD>Sends the startTransaction command to the NeoCore XMS.</DD></DL>
     * @doc <br/>Sends the startTransaction command to the NeoCore XMS. 
     * This method cannot be invoked with a null or blank sid.   
     * 
     * @param flushOnCommit - If true then flush on commit, otherwise don't flush on commit
     * @exception NeoServerException - If there was an internal NeoCore XMS error.
     * @exception NeoCommunicationException If there was an error communicating with the host.
     * @exception NeoTransactionException If any transactional errors occurred.
     * @exception NeoInvalidSessionException - If this objects session ID is not valid.
     * @exception NeoAuthorizationFailedException - If you do not have authorization to make this request.
     * @exception NeoPoolClosedException - If this object is using a pool, and that pool was closed.
     * @exception NeoConnectionPoolException - If this object is using a pool, and no connection were available from that pool.
     * @return XML, NeoCore XMS's response.
     */
	char* startTransaction (const bool flushOnCommit) 
    throw  (NeoServerException,
            NeoCommunicationException,
            NeoTransactionException,
            NeoInvalidSessionException,
            NeoAuthorizationFailedException,
            NeoPoolClosedException,
            NeoConnectionPoolException); 


    /**
     * @memo <DL><DD>Sends the startTransaction command to the NeoCore XMS.</DD></DL>
     * @doc <br/>Sends the startTransaction command to the NeoCore XMS. 
     * This method cannot be invoked with a null or blank sid.
     *
     * @param maxDuration -  The number of seconds that are allowed between this startTransaction and
     * a subsequent commitTransaction. If this duration exceeds maxDuration then this transaction is
     * automatically rolled back. If the provided value is invalid <= 0 then the default is used.
     * @param inactivityDuration -  The number of seconds that this transaction can be inactive. 
	 * If this duration exceeds inactivityDuration then this transaction is
     * automatically rolled back. If the provided value is invalid <= 0 then the default is used.     
	 * @param flushOnCommit - If true then flush on commit, otherwise don't flush on commit
     * @exception NeoServerException - If there was an internal NeoCore XMS error.
     * @exception NeoCommunicationException - If there was an error communicating with the host.
     * @exception NeoTransactionException - If any transactional errors occurred.
     * @exception NeoInvalidSessionException - If this objects session ID is not valid.
     * @exception NeoAuthorizationFailedException - If you do not have authorization to make this request					
     * @exception NeoPoolClosedException - If this object is using a pool, and that pool was closed.
     * @exception NeoConnectionPoolException - If this object is using a pool, and no connection were available from that pool.
     * @return XML, NeoCore XMS's response
     */
     char* startTransaction(const int maxDuration,
							const int inactivityDuration,
							const bool flushOnCommit) 
    throw  (NeoServerException,
            NeoCommunicationException,
            NeoTransactionException,
            NeoInvalidSessionException,
            NeoAuthorizationFailedException,
            NeoPoolClosedException,
            NeoConnectionPoolException);


   /**
     * @memo <DL><DD>Sends the rollback command to the NeoCore XMS.</DD></DL>
     * @doc <br/>Sends the rollback command to the NeoCore XMS.
     *
     * @exception NeoServerException - If there was an internal NeoCore XMS error.
     * @exception NeoCommunicationException - If there was an error communicating with the host.
     * @exception NeoTransactionException - If any transactional errors occurred.
     * @exception NeoInvalidSessionException - If this objects session ID is not valid.
     * @exception NeoAuthorizationFailedException - If you do not have authorization to make this request.
     * @exception NeoPoolClosedException - If this object is using a pool, and that pool was closed.
     * @exception NeoConnectionPoolException - If this object is using a pool, and no connection were available from that pool.
     * @return XML, NeoCore XMS's response.     
     */
	char* rollbackTransaction () 
    throw (NeoServerException,
           NeoCommunicationException,
           NeoTransactionException,
           NeoInvalidSessionException,
           NeoAuthorizationFailedException,
           NeoPoolClosedException,
           NeoConnectionPoolException);




   /**
     * @memo <DL><DD>Release the result buffer.</DD></DL>
     * @doc <br/>Release the result buffer.  All buffers returned from NeoConnection must be released. 
	 * @return void
	 */
	void releaseBuffer (char* cbuffer);



	/** 
	  * @memo <DL><DD>Release the NeoQueryResult.</DD></DL>
	  * @doc <br/>Release the NeoQueryResult.   All results returned must be released.
      * 
	  * @param neoQueryResult  the NeoQueryResult you want to release 
	  */
	void  releaseNeoQueryResult(NeoQueryResult* neoQueryResult);





    /**
	 * @memo <DL><DD>This sends a set isolation level command to NeoCore XMS.</DD></DL>
     * @doc <br/>This sends a set isolation level command to NeoCore XMS.
     * These levels correspond to SQL2's standard.
     * READ_UNCOMMITTED - allow dirty reads.  No read locks are used. 
     * READ_COMMITTED - Read and write locks are used.  Read locks are released after each command.
     * REPEATABLE_READ - Read and write locks are used.  Read locks are not released until end of transaction.
     * No protection against phantoms.<br>
     * The server is defaulted to REPEATABLE_READ upon the creation of a new session.
     * This is valid for the duration of the current session.
     *
     * @param level The isolation level to use from one of the acceptable levels.
     * @exception NeoServerException - If there was an internal server error.
     * @exception NeoCommunicationException - If there was an error communicating with the host.
     * @exception NeoTransactionException - If any transactional errors occurred.
     * @exception NeoInvalidArgumentException - If the level is blank, NULL or not a valid level.
     * @exception NeoInvalidSessionException - If the provided session ID is not valid.
     * @exception NeoAuthorizationFailedException - If you do not have authorization to make this request.
     * @exception NeoPoolClosedException - If this object is using a pool, and that pool was closed.
     * @exception NeoConnectionPoolException - If this object is using a pool, and no connection were available from that pool.
     * @return XML containing the isolation level after being set.
	 */
	char* setIsolationLevel (const char* level)    
    throw  (NeoServerException,  
            NeoCommunicationException,
            NeoTransactionException,
            NeoInvalidArgumentException,
            NeoInvalidSessionException,
            NeoAuthorizationFailedException,
            NeoPoolClosedException,
            NeoConnectionPoolException);

	/**
	* @memo <DL><DD><B><I>DEPRECATED</I></B> - This is for Solaris IPlanet support only.</DD></DL>
	* @doc <br/><B><I>DEPRECATED</I></B> - This is for Solaris IPlanet support only.
	*
	* @param flag If this is set to TRUE the server is started, 
	* if FALSE the server is stopped.  
	* @param key  Administratore key required to execute the command. 
    * @exception NeoInvalidArgumentException - If adminKey argument is empty or null.
	* @return true if successful, otherwise returns false.
	*
	*/
	bool ControlServer(const bool flag, const char *adminKey) 
	throw (NeoInvalidArgumentException);


	char* getSID();
};
#endif 

